@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: config.c,v 1.119 1999/12/20 13:07:41 karls Exp $";
+"$Id: config.c,v 1.124 2000/06/09 10:45:17 karls Exp $";
 
 __BEGIN_DECLS
 
@@ -118,7 +118,7 @@ genericinit(void)
 		return;
 #endif
 
-	initlog();
+	newprocinit();
 
 #if !HAVE_NO_RESOLVESTUFF
 	res_init();
@@ -586,7 +586,7 @@ addroute(newroute)
 	struct route_t *route;
 
 	if ((route = (struct route_t *)malloc(sizeof(*route))) == NULL)
-		serrx(1, "%s: %s", function, NOMEM);
+		serrx(EXIT_FAILURE, "%s: %s", function, NOMEM);
 	*route = *newroute;
 
 	/* check gateway. */
@@ -1039,7 +1039,6 @@ void
 showstate(state)
 	const struct serverstate_t *state;
 {
-	int i;
 	char buf[1024];
 	size_t bufused;
 
@@ -1075,11 +1074,7 @@ showstate(state)
 		PROTOCOL_UDPs);
 	slog(LOG_INFO, buf);
 
-	bufused = snprintf(buf, sizeof(buf), "method(s): ");
-	for (i = 0; i < state->methodc; ++i)
-		bufused += snprintf(&buf[bufused], sizeof(buf) - bufused, "%s, ",
-		method2string(state->methodv[i]));
-	slog(LOG_INFO, buf);
+	showmethod(state->methodc, state->methodv);
 
 	bufused = snprintf(buf, sizeof(buf), "proxyprotocol(s): ");
 	if (state->proxyprotocol.socks_v4)
@@ -1090,3 +1085,19 @@ showstate(state)
 		bufused += snprintf(&buf[bufused], sizeof(buf) - bufused, "msproxy v2");
 	slog(LOG_INFO, buf);
 }
+
+void
+showmethod(methodc, methodv)
+	int methodc;
+	const int *methodv;
+{
+	int i;
+	char buf[1024];
+	size_t bufused;
+
+	bufused = snprintf(buf, sizeof(buf), "method(s): ");
+	for (i = 0; i < methodc; ++i)
+		bufused += snprintf(&buf[bufused], sizeof(buf) - bufused, "%s, ",
+		method2string(methodv[i]));
+	slog(LOG_INFO, buf);
+}
