@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -47,15 +47,15 @@
 
 #if SOCKSLIBRARY_DYNAMIC
 
-#include "interposition.h"
 
 static const char rcsid[] =
-"$Id: interposition.c,v 1.53 1999/09/25 13:10:31 karls Exp $";
+"$Id: interposition.c,v 1.71 2000/06/21 08:48:16 michaels Exp $";
 
 #undef accept
 #undef bind
 #undef bindresvport
 #undef connect
+#undef gethostbyaddr
 #undef gethostbyname
 #undef gethostbyname2
 #undef getpeername
@@ -77,44 +77,53 @@ static const char rcsid[] =
 #endif  /* NEED_DYNA_RTLD */
 
 static struct libsymbol_t libsymbolv[] = {
-	{	SYMBOL_ACCEPT,						LIBRARY_ACCEPT },
-	{	SYMBOL_BIND,						LIBRARY_BIND },
-	{	SYMBOL_BINDRESVPORT,				LIBRARY_BINDRESVPORT },
-	{	SYMBOL_CONNECT,					LIBRARY_CONNECT },
-	{	SYMBOL_GETHOSTBYNAME,			LIBRARY_GETHOSTBYNAME },
-	{	SYMBOL_GETHOSTBYNAME2,			LIBRARY_GETHOSTBYNAME2 },
-	{	SYMBOL_GETPEERNAME,				LIBRARY_GETPEERNAME },
-	{	SYMBOL_GETSOCKNAME,				LIBRARY_GETSOCKNAME },
-	{	SYMBOL_READ,						LIBRARY_READ },
-	{	SYMBOL_READV,						LIBRARY_READV },
-	{	SYMBOL_RECV,						LIBRARY_RECV },
-	{	SYMBOL_RECVMSG,					LIBRARY_RECVMSG },
-	{	SYMBOL_RECVFROM,					LIBRARY_RECVFROM },
-	{	SYMBOL_RRESVPORT,					LIBRARY_RRESVPORT },
-	{	SYMBOL_SEND,						LIBRARY_SEND },
-	{	SYMBOL_SENDMSG,					LIBRARY_SENDMSG },
-	{	SYMBOL_SENDTO,						LIBRARY_SENDTO },
-	{	SYMBOL_WRITE,						LIBRARY_WRITE },
-	{	SYMBOL_WRITEV,						LIBRARY_WRITEV },
+#if SOCKS_CLIENT
+	{	SYMBOL_ACCEPT,						LIBRARY_ACCEPT,			NULL,	NULL },
+	{	SYMBOL_BIND,						LIBRARY_BIND,				NULL,	NULL },
+	{	SYMBOL_BINDRESVPORT,				LIBRARY_BINDRESVPORT,	NULL,	NULL },
+	{	SYMBOL_CONNECT,					LIBRARY_CONNECT,			NULL,	NULL },
+	{	SYMBOL_GETPEERNAME,				LIBRARY_GETPEERNAME,		NULL,	NULL },
+	{	SYMBOL_GETSOCKNAME,				LIBRARY_GETSOCKNAME,		NULL,	NULL },
+	{	SYMBOL_READ,						LIBRARY_READ,				NULL,	NULL },
+	{	SYMBOL_READV,						LIBRARY_READV,				NULL,	NULL },
+	{	SYMBOL_RECV,						LIBRARY_RECV,				NULL,	NULL },
+	{	SYMBOL_RECVMSG,					LIBRARY_RECVMSG,			NULL,	NULL },
+	{	SYMBOL_RECVFROM,					LIBRARY_RECVFROM,			NULL,	NULL },
+	{	SYMBOL_RRESVPORT,					LIBRARY_RRESVPORT,		NULL,	NULL },
+	{	SYMBOL_SEND,						LIBRARY_SEND,				NULL,	NULL },
+	{	SYMBOL_SENDMSG,					LIBRARY_SENDMSG,			NULL,	NULL },
+	{	SYMBOL_SENDTO,						LIBRARY_SENDTO,			NULL,	NULL },
+	{	SYMBOL_WRITE,						LIBRARY_WRITE,				NULL,	NULL },
+	{	SYMBOL_WRITEV,						LIBRARY_WRITEV,			NULL,	NULL },
+#if HAVE_GETHOSTBYNAME2
+	{	SYMBOL_GETHOSTBYNAME2,			LIBRARY_GETHOSTBYNAME2,	NULL,	NULL },
+#endif /* HAVE_GETHOSTBYNAME2 */
+#endif /* SOCKS_CLIENT */
+	{	SYMBOL_GETHOSTBYNAME,			LIBRARY_GETHOSTBYNAME,	NULL,	NULL },
+
+#if SOCKS_SERVER
+	{	SYMBOL_GETHOSTBYADDR,			LIBRARY_GETHOSTBYADDR,	NULL,	NULL },
+#endif
+
+#if SOCKS_CLIENT
 #if HAVE_EXTRA_OSF_SYMBOLS
-	{	SYMBOL_EACCEPT,					LIBRARY_EACCEPT },
-	{	SYMBOL_EGETPEERNAME,				LIBRARY_EGETPEERNAME },
-	{  SYMBOL_EGETSOCKNAME,				LIBRARY_EGETSOCKNAME },
-	{	SYMBOL_EREADV,						LIBRARY_EREADV },
-	{	SYMBOL_ERECVFROM,					LIBRARY_ERECVFROM },
-	{	SYMBOL_ERECVMSG,					LIBRARY_ERECVMSG },
-	{  SYMBOL_ESENDMSG,					LIBRARY_ESENDMSG },
-	{	SYMBOL_EWRITEV,					LIBRARY_EWRITEV },
+	{	SYMBOL_EACCEPT,					LIBRARY_EACCEPT,			NULL,	NULL },
+	{	SYMBOL_EGETPEERNAME,				LIBRARY_EGETPEERNAME,	NULL,	NULL },
+	{  SYMBOL_EGETSOCKNAME,				LIBRARY_EGETSOCKNAME,	NULL,	NULL },
+	{	SYMBOL_EREADV,						LIBRARY_EREADV,			NULL,	NULL },
+	{	SYMBOL_ERECVFROM,					LIBRARY_ERECVFROM,		NULL,	NULL },
+	{	SYMBOL_ERECVMSG,					LIBRARY_ERECVMSG,			NULL,	NULL },
+	{  SYMBOL_ESENDMSG,					LIBRARY_ESENDMSG,			NULL,	NULL },
+	{	SYMBOL_EWRITEV,					LIBRARY_EWRITEV,			NULL,	NULL },
 
-	{	SYMBOL_NACCEPT,					LIBRARY_EACCEPT },
-	{	SYMBOL_NGETPEERNAME,				LIBRARY_NGETPEERNAME },
-	{	SYMBOL_NGETSOCKNAME,				LIBRARY_NGETSOCKNAME },
-	{  SYMBOL_NRECVFROM,					LIBRARY_NRECVFROM },
-	{  SYMBOL_NRECVMSG,					LIBRARY_NRECVMSG },
-	{	SYMBOL_NSENDMSG,					LIBRARY_NSENDMSG },
+	{	SYMBOL_NACCEPT,					LIBRARY_EACCEPT,			NULL,	NULL },
+	{	SYMBOL_NGETPEERNAME,				LIBRARY_NGETPEERNAME,	NULL,	NULL },
+	{	SYMBOL_NGETSOCKNAME,				LIBRARY_NGETSOCKNAME,	NULL,	NULL },
+	{  SYMBOL_NRECVFROM,					LIBRARY_NRECVFROM,		NULL,	NULL },
+	{  SYMBOL_NRECVMSG,					LIBRARY_NRECVMSG,			NULL,	NULL },
+	{	SYMBOL_NSENDMSG,					LIBRARY_NSENDMSG,			NULL,	NULL },
 #endif  /* HAVE_EXTRA_OSF_SYMBOLS */
-
-
+#endif /* SOCKS_CLIENT */
 };
 
 __BEGIN_DECLS
@@ -127,19 +136,13 @@ libsymbol __P((const char *symbol));
 
 __END_DECLS
 
-static struct libsymbol_t *
-libsymbol(symbol)
-	const char *symbol;
+void
+symbolcheck(void)
 {
-	const char *function = "libsymbol()";
 	size_t i;
 
 	for (i = 0; i < ELEMENTS(libsymbolv); ++i)
-		if (strcmp(libsymbolv[i].symbol, symbol) == 0)
-			return &libsymbolv[i];
-
-	serrx(1, "%s: configuration error, can't find symbol %s", function, symbol);
-	return NULL; /* please compiler. */
+		symbolfunction(libsymbolv[i].symbol);
 }
 
 
@@ -158,11 +161,15 @@ symbolfunction(symbol)
 
 	if (lib->handle == NULL)
 		if ((lib->handle = dlopen(lib->library, DL_LAZY)) == NULL)
-			serrx(EXIT_FAILURE, "%s: %s: %s", function, lib->library, dlerror());
+			serrx(EXIT_FAILURE, "%s: compiletime configuration error?  "
+			"Failed to open \"%s\": %s",
+			function, lib->library, dlerror());
 
 	if (lib->function == NULL)
 		if ((lib->function = dlsym(lib->handle, symbol)) == NULL)
-			serrx(EXIT_FAILURE, "%s: %s: %s", function, symbol, dlerror());
+			serrx(EXIT_FAILURE, "%s: compiletime configuration error?  "
+			"Failed to find \"%s\" in \"%s\": %s",
+			function, symbol, lib->library, dlerror());
 
 #if 0
 	if (strcmp(symbol, SYMBOL_WRITE) != 0)
@@ -173,9 +180,29 @@ symbolfunction(symbol)
 	return lib->function;
 }
 
+static struct libsymbol_t *
+libsymbol(symbol)
+	const char *symbol;
+{
+/*	const char *function = "libsymbol()"; */
+	size_t i;
+
+	for (i = 0; i < ELEMENTS(libsymbolv); ++i)
+		if (strcmp(libsymbolv[i].symbol, symbol) == 0)
+			return &libsymbolv[i];
+
+	SASSERTX(0);	/* should never happen. */
+
+	/* NOTREACHED */
+	return NULL; /* please compiler. */
+}
+
+
 
 	/* the real system calls. */
 
+#if SOCKS_CLIENT
+
 #if !HAVE_EXTRA_OSF_SYMBOLS
 HAVE_PROT_ACCEPT_0
 sys_accept(s, addr, addrlen)
@@ -184,10 +211,13 @@ sys_accept(s, addr, addrlen)
 	HAVE_PROT_ACCEPT_3 addrlen;
 {
 	int rc;
-	HAVE_PROT_ACCEPT_0 (*function)(HAVE_PROT_ACCEPT_1 s, HAVE_PROT_ACCEPT_2 addr, HAVE_PROT_ACCEPT_3 addrlen);
+	typedef HAVE_PROT_ACCEPT_0 (*ACCEPT_FUNC_T)(HAVE_PROT_ACCEPT_1,
+															  HAVE_PROT_ACCEPT_2,
+															  HAVE_PROT_ACCEPT_3);
+	ACCEPT_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_ACCEPT);
+	function = (ACCEPT_FUNC_T)symbolfunction(SYMBOL_ACCEPT);
 	rc = function(s, addr, addrlen);
 	SYSCALL_END(s);
 	return rc;
@@ -202,10 +232,13 @@ sys_bind(s, name, namelen)
 	HAVE_PROT_BIND_3 namelen;
 {
 	int rc;
-	HAVE_PROT_BIND_0 (*function)(HAVE_PROT_BIND_1 s, HAVE_PROT_BIND_2 name, HAVE_PROT_BIND_3 namelen);
+	typedef HAVE_PROT_BIND_0 (*BIND_FUNC_T)(HAVE_PROT_BIND_1,
+														 HAVE_PROT_BIND_2,
+														 HAVE_PROT_BIND_3);
+	BIND_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_BIND);
+	function = (BIND_FUNC_T)symbolfunction(SYMBOL_BIND);
 	rc = function(s, name, namelen);
 	SYSCALL_END(s);
 	return rc;
@@ -218,10 +251,11 @@ sys_bindresvport(sd, sin)
 	struct sockaddr_in *sin;
 {
 	int rc;
-	int (*function)(int sd, struct sockaddr_in *sin);
+	typedef int (*BINDRESVPORT_FUNC_T)(int, struct sockaddr_in *);
+	BINDRESVPORT_FUNC_T function;
 
 	SYSCALL_START(sd);
-	function = symbolfunction(SYMBOL_BINDRESVPORT);
+	function = (BINDRESVPORT_FUNC_T)symbolfunction(SYMBOL_BINDRESVPORT);
 	rc = function(sd, sin);
 	SYSCALL_END(sd);
 	return rc;
@@ -235,37 +269,19 @@ sys_connect(s, name, namelen)
 	HAVE_PROT_CONNECT_3 namelen;
 {
 	int rc;
-	HAVE_PROT_CONNECT_0 (*function)(HAVE_PROT_CONNECT_1 s, HAVE_PROT_CONNECT_2 name, HAVE_PROT_CONNECT_3 namelen);
+	typedef HAVE_PROT_CONNECT_0 (*CONNECT_FUNC_T)(HAVE_PROT_CONNECT_1,
+																 HAVE_PROT_CONNECT_2,
+																 HAVE_PROT_CONNECT_3);
+	CONNECT_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_CONNECT);
+	function = (CONNECT_FUNC_T)symbolfunction(SYMBOL_CONNECT);
 	rc = function(s, name, namelen);
 	SYSCALL_END(s);
 	return rc;
 }
 #endif /* !HAVE_EXTRA_OSF_SYMBOLS */
 
-struct hostent *
-sys_gethostbyname(name)
-	const char *name;
-{
-	struct hostent *(*function)(const char *name);
-
-	function = symbolfunction(SYMBOL_GETHOSTBYNAME);
-	return function(name);
-}
-
-struct hostent *
-sys_gethostbyname2(name, af)
-	const char *name;
-	int af;
-{
-	struct hostent *(*function)(const char *name, int af);
-
-	function = symbolfunction(SYMBOL_GETHOSTBYNAME2);
-	return function(name, af);
-}
-
 #if !HAVE_EXTRA_OSF_SYMBOLS
 HAVE_PROT_GETPEERNAME_0
 sys_getpeername(s, name, namelen)
@@ -274,10 +290,14 @@ sys_getpeername(s, name, namelen)
 	HAVE_PROT_GETPEERNAME_3 namelen;
 {
 	int rc;
-	HAVE_PROT_GETPEERNAME_0 (*function)(HAVE_PROT_GETPEERNAME_1 s, HAVE_PROT_GETPEERNAME_2 name, HAVE_PROT_GETPEERNAME_3 namelen);
+	typedef HAVE_PROT_GETPEERNAME_0
+		 (*GETPEERNAME_FUNC_T)(HAVE_PROT_GETPEERNAME_1,
+									  HAVE_PROT_GETPEERNAME_2,
+									  HAVE_PROT_GETPEERNAME_3);
+	GETPEERNAME_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_GETPEERNAME);
+	function = (GETPEERNAME_FUNC_T)symbolfunction(SYMBOL_GETPEERNAME);
 	rc = function(s, name, namelen);
 	SYSCALL_END(s);
 	return rc;
@@ -292,10 +312,14 @@ sys_getsockname(s, name, namelen)
 	HAVE_PROT_GETSOCKNAME_3 namelen;
 {
 	int rc;
-	HAVE_PROT_GETSOCKNAME_0 (*function)(HAVE_PROT_GETSOCKNAME_1 s, HAVE_PROT_GETSOCKNAME_2 name, HAVE_PROT_GETSOCKNAME_3 namelen);
+	typedef HAVE_PROT_GETSOCKNAME_0
+		 (*GETSOCKNAME_FUNC_T)(HAVE_PROT_GETSOCKNAME_1,
+									  HAVE_PROT_GETSOCKNAME_2,
+									  HAVE_PROT_GETSOCKNAME_3);
+	GETSOCKNAME_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_GETSOCKNAME);
+	function = (GETSOCKNAME_FUNC_T)symbolfunction(SYMBOL_GETSOCKNAME);
 	rc = function(s, name, namelen);
 	SYSCALL_END(s);
 	return rc;
@@ -309,10 +333,13 @@ sys_read(d, buf, nbytes)
 	HAVE_PROT_READ_3 nbytes;
 {
 	ssize_t rc;
-	HAVE_PROT_READ_0 (*function)(HAVE_PROT_READ_1 d, HAVE_PROT_READ_2 buf, HAVE_PROT_READ_3 nbytes);
+	typedef HAVE_PROT_READ_0 (*READ_FUNC_T)(HAVE_PROT_READ_1,
+														 HAVE_PROT_READ_2,
+														 HAVE_PROT_READ_3);
+	READ_FUNC_T function;
 
 	SYSCALL_START(d);
-	function = symbolfunction(SYMBOL_READ);
+	function = (READ_FUNC_T)symbolfunction(SYMBOL_READ);
 	rc = function(d, buf, nbytes);
 	SYSCALL_END(d);
 	return rc;
@@ -326,10 +353,13 @@ sys_readv(d, iov, iovcnt)
 	HAVE_PROT_READV_3 iovcnt;
 {
 	ssize_t rc;
-	HAVE_PROT_READV_0 (*function)(HAVE_PROT_READV_1 d, HAVE_PROT_READV_2 iov, HAVE_PROT_READV_3 iovcnt);
+	typedef HAVE_PROT_READV_0 (*READV_FUNC_T)(HAVE_PROT_READV_1,
+															HAVE_PROT_READV_2,
+															HAVE_PROT_READV_3);
+	READV_FUNC_T function;
 
 	SYSCALL_START(d);
-	function = symbolfunction(SYMBOL_READV);
+	function = (READV_FUNC_T)symbolfunction(SYMBOL_READV);
 	rc = function(d, iov, iovcnt);
 	SYSCALL_END(d);
 	return rc;
@@ -344,10 +374,14 @@ sys_recv(s, buf, len, flags)
 	HAVE_PROT_RECV_4 flags;
 {
 	ssize_t rc;
-	HAVE_PROT_RECV_0 (*function)(HAVE_PROT_RECV_1 s, HAVE_PROT_RECV_2 buf, HAVE_PROT_RECV_3 len, HAVE_PROT_RECV_4 flags);
+	typedef HAVE_PROT_RECV_0 (*RECV_FUNC_T)(HAVE_PROT_RECV_1,
+														 HAVE_PROT_RECV_2,
+														 HAVE_PROT_RECV_3,
+														 HAVE_PROT_RECV_4);
+	RECV_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_RECV);
+	function = (RECV_FUNC_T)symbolfunction(SYMBOL_RECV);
 	rc = function(s, buf, len, flags);
 	SYSCALL_END(s);
 	return rc;
@@ -364,10 +398,16 @@ sys_recvfrom(s, buf, len, flags, from, fromlen)
 	HAVE_PROT_RECVFROM_6 fromlen;
 {
 	int rc;
-	HAVE_PROT_RECVFROM_0 (*function)(HAVE_PROT_RECVFROM_1 s, HAVE_PROT_RECVFROM_2 buf, HAVE_PROT_RECVFROM_3 len, HAVE_PROT_RECVFROM_4 flags, HAVE_PROT_RECVFROM_5 from, HAVE_PROT_RECVFROM_6 fromlen);
+	typedef HAVE_PROT_RECVFROM_0 (*RECVFROM_FUNC_T)(HAVE_PROT_RECVFROM_1,
+																	HAVE_PROT_RECVFROM_2,
+																	HAVE_PROT_RECVFROM_3,
+																	HAVE_PROT_RECVFROM_4,
+																	HAVE_PROT_RECVFROM_5,
+																	HAVE_PROT_RECVFROM_6);
+	RECVFROM_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_RECVFROM);
+	function = (RECVFROM_FUNC_T)symbolfunction(SYMBOL_RECVFROM);
 	rc = function(s, buf, len, flags, from, fromlen);
 	SYSCALL_END(s);
 	return rc;
@@ -382,10 +422,13 @@ sys_recvmsg(s, msg, flags)
 	HAVE_PROT_RECVMSG_3 flags;
 {
 	ssize_t rc;
-	HAVE_PROT_RECVMSG_0 (*function)(HAVE_PROT_RECVMSG_1 s, HAVE_PROT_RECVMSG_2 msg, HAVE_PROT_RECVMSG_3 flags);
+	typedef HAVE_PROT_RECVMSG_0 (*RECVMSG_FUNC_T)(HAVE_PROT_RECVMSG_1,
+																 HAVE_PROT_RECVMSG_2,
+																 HAVE_PROT_RECVMSG_3);
+	RECVMSG_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_RECVMSG);
+	function = (RECVMSG_FUNC_T)symbolfunction(SYMBOL_RECVMSG);
 	rc = function(s, msg, flags);
 	SYSCALL_END(s);
 	return rc;
@@ -396,9 +439,10 @@ int
 sys_rresvport(port)
 	int *port;
 {
-	int (*function)(int *port);
+	typedef int (*RRESVPORT_FUNC_T)(int *);
+	RRESVPORT_FUNC_T function;
 
-	function = symbolfunction(SYMBOL_RRESVPORT);
+	function = (RRESVPORT_FUNC_T)symbolfunction(SYMBOL_RRESVPORT);
 	return function(port);
 }
 
@@ -410,7 +454,11 @@ sys_send(s, msg, len, flags)
 	HAVE_PROT_SEND_4 flags;
 {
 	ssize_t rc;
-	HAVE_PROT_SEND_0 (*function)(HAVE_PROT_SEND_1 s, HAVE_PROT_SEND_2 msg, HAVE_PROT_SEND_3 len, HAVE_PROT_SEND_4 flags);
+	typedef HAVE_PROT_SEND_0 (*SEND_FUNC_T)(HAVE_PROT_SEND_1,
+														 HAVE_PROT_SEND_2,
+														 HAVE_PROT_SEND_3,
+														 HAVE_PROT_SEND_4);
+	SEND_FUNC_T function;
 
 	SYSCALL_START(s);
 	function = symbolfunction(SYMBOL_SEND);
@@ -427,10 +475,13 @@ sys_sendmsg(s, msg, flags)
 	HAVE_PROT_SENDMSG_3 flags;
 {
 	ssize_t rc;
-	HAVE_PROT_SENDMSG_0 (*function)(HAVE_PROT_SENDMSG_1 s, HAVE_PROT_SENDMSG_2 msg, HAVE_PROT_SENDMSG_3  flags);
+	typedef HAVE_PROT_SENDMSG_0 (*SENDMSG_FUNC_T)(HAVE_PROT_SENDMSG_1,
+																 HAVE_PROT_SENDMSG_2,
+																 HAVE_PROT_SENDMSG_3);
+	SENDMSG_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_SENDMSG);
+	function = (SENDMSG_FUNC_T)symbolfunction(SYMBOL_SENDMSG);
 	rc = function(s, msg, flags);
 	SYSCALL_END(s);
 	return rc;
@@ -448,10 +499,16 @@ sys_sendto(s, msg, len, flags, to, tolen)
 	HAVE_PROT_SENDTO_6 tolen;
 {
 	ssize_t rc;
-	HAVE_PROT_SENDTO_0 (*function)(HAVE_PROT_SENDTO_1 s, HAVE_PROT_SENDTO_2 msg, HAVE_PROT_SENDTO_3 len, HAVE_PROT_SENDTO_4 flags, HAVE_PROT_SENDTO_5 to, HAVE_PROT_SENDTO_6 tolen);
+	typedef HAVE_PROT_SENDTO_0 (*SENDTO_FUNC_T)(HAVE_PROT_SENDTO_1,
+															  HAVE_PROT_SENDTO_2,
+															  HAVE_PROT_SENDTO_3,
+															  HAVE_PROT_SENDTO_4,
+															  HAVE_PROT_SENDTO_5,
+															  HAVE_PROT_SENDTO_6);
+	SENDTO_FUNC_T function;
 
 	SYSCALL_START(s);
-	function = symbolfunction(SYMBOL_SENDTO);
+	function = (SENDTO_FUNC_T)symbolfunction(SYMBOL_SENDTO);
 	rc = function(s, msg, len, flags, to, tolen);
 	SYSCALL_END(s);
 	return rc;
@@ -465,10 +522,13 @@ sys_write(d, buf, nbytes)
 	HAVE_PROT_WRITE_3 nbytes;
 {
 	ssize_t rc;
-	HAVE_PROT_WRITE_0 (*function)(HAVE_PROT_WRITE_1 d, HAVE_PROT_WRITE_2 buf, HAVE_PROT_WRITE_3 nbutes);
+	typedef HAVE_PROT_WRITE_0 (*WRITE_FUNC_T)(HAVE_PROT_WRITE_1,
+															HAVE_PROT_WRITE_2,
+															HAVE_PROT_WRITE_3);
+	WRITE_FUNC_T function;
 
 	SYSCALL_START(d);
-	function = symbolfunction(SYMBOL_WRITE);
+	function = (WRITE_FUNC_T)symbolfunction(SYMBOL_WRITE);
 	rc = function(d, buf, nbytes);
 	SYSCALL_END(d);
 	return rc;
@@ -482,10 +542,13 @@ sys_writev(d, iov, iovcnt)
 	HAVE_PROT_WRITEV_3 iovcnt;
 {
 	ssize_t rc;
-	HAVE_PROT_WRITEV_0 (*function)(HAVE_PROT_WRITEV_1 d, HAVE_PROT_WRITEV_2 buf, HAVE_PROT_WRITEV_3 iovcnt);
+	typedef HAVE_PROT_WRITEV_0 (*WRITEV_FUNC_T)(HAVE_PROT_WRITEV_1,
+															  HAVE_PROT_WRITEV_2,
+															  HAVE_PROT_WRITEV_3);
+	WRITEV_FUNC_T function;
 
 	SYSCALL_START(d);
-	function = symbolfunction(SYMBOL_WRITEV);
+	function = (WRITEV_FUNC_T)symbolfunction(SYMBOL_WRITEV);
 	rc = function(d, iov, iovcnt);
 	SYSCALL_END(d);
 	return rc;
@@ -506,7 +569,7 @@ accept(s, addr, addrlen)
 {
 	if (ISSYSCALL(s))
 		return sys_accept(s, addr, addrlen);
-	return Raccept(s, addr, addrlen);
+	return Raccept(s, addr, (socklen_t *)addrlen);
 }
 #endif /* HAVE_EXTRA_OSF_SYMBOLS */
 
@@ -546,21 +609,6 @@ connect(s, name, namelen)
 }
 #endif /* HAVE_EXTRA_OSF_SYMBOLS */
 
-struct hostent *
-gethostbyname(name)
-	const char *name;
-{
-	return Rgethostbyname(name);
-}
-
-struct hostent *
-gethostbyname2(name, af)
-	const char *name;
-	int af;
-{
-	return Rgethostbyname2(name, af);
-}
-
 
 #if !HAVE_EXTRA_OSF_SYMBOLS
 HAVE_PROT_GETPEERNAME_0
@@ -583,7 +631,7 @@ getsockname(s, name, namelen)
 	HAVE_PROT_GETSOCKNAME_3 namelen;
 {
 	if (ISSYSCALL(s))
-		return sys_getpeername(s, name, namelen);
+		return sys_getsockname(s, name, namelen);
 	return Rgetsockname(s, name, namelen);
 }
 #endif /* HAVE_EXTRA_OSF_SYMBOLS */
@@ -725,4 +773,80 @@ sendto(s, msg, len, flags, to, tolen)
 }
 #endif /* !HAVE_EXTRA_OSF_SYMBOLS */
 
+#endif /* SOCKS_CLIENT */
+
+#if SOCKS_SERVER
+
+struct hostent *
+sys_gethostbyaddr(addr, len, af)
+	const char *addr;
+	int len;
+	int af;
+{
+	typedef struct hostent *(*GETHOSTBYADDR_FUNC_T)(const char *, int, int);
+	GETHOSTBYADDR_FUNC_T function;
+
+	function = (GETHOSTBYADDR_FUNC_T)symbolfunction(SYMBOL_GETHOSTBYADDR);
+	return function(addr, len, af);
+}
+
+HAVE_PROT_GETHOSTBYADDR_0
+gethostbyaddr(addr, len, af)
+	HAVE_PROT_GETHOSTBYADDR_1 addr;
+	HAVE_PROT_GETHOSTBYADDR_2 len;
+	HAVE_PROT_GETHOSTBYADDR_3 af;
+{
+
+	return cgethostbyaddr(addr, len, af);
+}
+
+#endif /* SOCKS_SERVER */
+
+struct hostent *
+sys_gethostbyname(name)
+	const char *name;
+{
+	typedef struct hostent *(*GETHOSTBYNAME_FUNC_T)(const char *);
+	GETHOSTBYNAME_FUNC_T function;
+
+	function = (GETHOSTBYNAME_FUNC_T)symbolfunction(SYMBOL_GETHOSTBYNAME);
+	return function(name);
+}
+
+struct hostent *
+gethostbyname(name)
+	const char *name;
+{
+#if SOCKS_SERVER
+	return cgethostbyname(name);
+#else
+	return Rgethostbyname(name);
+#endif
+}
+
+#if SOCKS_CLIENT
+
+struct hostent *
+sys_gethostbyname2(name, af)
+	const char *name;
+	int af;
+{
+	typedef struct hostent *(*GETHOSTBYNAME2_FUNC_T)(const char *, int);
+	GETHOSTBYNAME2_FUNC_T function;
+
+	function = (GETHOSTBYNAME2_FUNC_T)symbolfunction(SYMBOL_GETHOSTBYNAME2);
+	return function(name, af);
+}
+
+
+struct hostent *
+gethostbyname2(name, af)
+	const char *name;
+	int af;
+{
+	return Rgethostbyname2(name, af);
+}
+
+#endif /* SOCKS_CLIENT */
+
 #endif /* SOCKSLIBRARY_DYNAMIC */
