@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,19 +41,33 @@
  *
  */
 
-/* $Id: common.h,v 1.218 1999/12/22 09:29:19 karls Exp $ */
+/* $Id: common.h,v 1.247 2000/06/09 10:45:16 karls Exp $ */
 
 #ifndef _COMMON_H_
 #define _COMMON_H_
 #endif
 
+#ifndef SOCKS_SERVER
+#define SOCKS_SERVER 0
+#endif /* !defined SOCKS_SERVER */
+
+#ifndef SOCKS_CLIENT
+#define SOCKS_CLIENT 0
+#endif /* !defined SOCKS_CLIENT */
+
+#ifndef SOCKSLIBRARY_DYNAMIC
+#define SOCKSLIBRARY_DYNAMIC 0
+#endif /* !defined SOCKSLIBRARY_DYNAMIC */
+
 /* XXX ifdef, not if, defined on command line */
 #ifdef HAVE_CONFIG_H
 #include "autoconf.h"
 #endif  /* HAVE_CONFIG_H */
 
 #include <sys/types.h>
+#if HAVE_SYS_TIME_H
 #include <sys/time.h>
+#endif /* HAVE_SYS_TIME_H */
 #if HAVE_SYS_FILE_H
 #include <sys/file.h>
 #endif  /* HAVE_SYS_FILE_H */
@@ -61,7 +75,7 @@
 #include <sys/ioctl.h>
 #include <sys/ipc.h>
 #include <sys/sem.h>
-/* XXX This is a hack. Avoid transparent sockaddr union used in linux
+/* XXX This is a hack. Avoid transparent sockaddr union used in Linux
    to avoid the use of the union in the code. Mainly used in
    interposition.c*/
 #ifdef WE_DONT_WANT_NO_SOCKADDR_ARG_UNION
@@ -97,7 +111,7 @@
 #include <crypt.h>
 #endif  /* HAVE_CRYPT_H */
 #include <ctype.h>
-#ifdef SOCKSLIBRARY_DYNAMIC
+#if SOCKSLIBRARY_DYNAMIC
 #include <dlfcn.h>
 #endif  /* SOCKSLIBRARY_DYNAMIC */
 #include <errno.h>
@@ -126,7 +140,9 @@
 #if HAVE_LIBWRAP && HAVE_TCPD_H
 #include <tcpd.h>
 #endif  /* HAVE_LIBWRAP && HAVE_TCPD_H */
+#if TIME_WITH_SYS_TIME || (!TIME_WITH_SYS_TIME && !HAVE_SYS_TIME_H)
 #include <time.h>
+#endif
 #if HAVE_UNISTD_H
 #ifdef HAVE_DEC_PROTO
 #undef  _XOPEN_SOURCE_EXTENDED
@@ -189,7 +205,7 @@ typedef int sig_atomic_t;
 #endif  /* !DIAGNOSTIC */
 
 /*
- * solaris 2.5.1 and it's stream stuff is broken and puts the processes
+ * Solaris 2.5.1 and it's stream stuff is broken and puts the processes
  * into never-never land forever on half the sendmsg() calls if they
  * involve ancillary data.  (it seems to deadlock the processes.)
  */
@@ -313,6 +329,22 @@ struct ipoption {
 };
 #endif  /* !HAVE_STRUCT_IPOPTS */
 
+#if 0
+#if !HAVE_SOCKADDR_STORAGE
+/*
+ * sockaddr_storage (see rfc2553).
+ * struct taken from OpenBSD <sys/socket.h>
+ */
+struct sockaddr_storage {
+	u_int8_t    ss_len;		/* total length */
+	sa_family_t ss_family;		/* address family */
+	u_char	    __ss_pad1[6];	/* align to quad */
+	u_int64_t   __ss_pad2;		/* force alignment for stupid compilers */
+	u_char      __ss_pad3[240];	/* pad to a total of 256 bytes */
+};
+#endif /* !HAVE_SOCKADDR_STORAGE */
+#endif
+
 /* global variables needed by everyone. */
 extern struct config_t config;
 extern char *__progname;
@@ -323,7 +355,7 @@ extern int h_errno;
 
 	/*
 	 * defines
-	*/
+	 */
 
 
 /*
@@ -356,6 +388,10 @@ extern int h_errno;
 
 #define	MAXRULEADDRSTRING	 (MAXSOCKSHOSTSTRING * 2)
 
+
+#define MAXAUTHINFOLEN		((sizeof("(") - 1) + MAXMETHODLEN) \
+									+ (sizeof(")" - 1) + (sizeof("@") - 1) + MAXNAMELEN)
+
 #ifndef NUL
 #define NUL '\0'
 #endif
@@ -390,7 +426,7 @@ extern int h_errno;
 
 	/*
 	 * macros
-	*/
+	 */
 
 
 #define close(n)	closen(n)
@@ -416,7 +452,7 @@ extern int h_errno;
 
 
 /*
- * macros to manipulate ancillary data depending on if were on sysv or bsd.
+ * macros to manipulate ancillary data depending on if we're on sysv or BSD.
  */
 
 /* allocate memory for data.  "size" is the amount of memory to allocate. */
@@ -584,7 +620,7 @@ do {														\
 	rcsid)
 
 
-/* the size of a udp header "packet" (no padding) */
+/* the size of a UDP header "packet" (no padding) */
 #define PACKETSIZE_UDP(packet) (												\
 	sizeof((packet)->flag) + sizeof((packet)->frag)						\
 	+ sizeof((packet)->host.atype) + sizeof((packet)->host.port)	\
@@ -614,10 +650,10 @@ do {														\
 
 /*
  * This is for Rgethostbyname() support for clients without access to
- * dns.
- * FAKEIP_START is the first address in the range of "fake" ip addresses,
+ * DNS.
+ * FAKEIP_START is the first address in the range of "fake" IP addresses,
  * FAKEIP_END is the last.
- * There can thus be FAKEIP_END - FAKEIP_START number of fake ip addresses
+ * There can thus be FAKEIP_END - FAKEIP_START number of fake IP addresses
  * supported per program.
  *
  * INADDR_NONE and INADDR_ANY may not be part of the range.
@@ -654,6 +690,11 @@ do {														\
 
 #define AUTHMETHOD_MAX			(AUTHMETHOD_RFC931 + 1)
 
+#define MAXMETHODLEN				MAX(sizeof(AUTHMETHOD_NONEs),		\
+										MAX(sizeof(AUTHMETHOD_GSSAPIs),	\
+										MAX(sizeof(AUTHMETHOD_UNAMEs),	\
+										sizeof(AUTHMETHOD_RFC931s))))
+
 /*
  *  Response commands/codes
  */
@@ -674,7 +715,7 @@ do {														\
 #define SOCKS_UDPREPLY				(SOCKS_BINDREPLY + 1)
 #define SOCKS_UDPREPLYs				"udpreply"
 
-/* misc stuff */
+/* misc. stuff */
 #define SOCKS_ACCEPT					(SOCKS_UDPREPLY + 1)
 #define SOCKS_ACCEPTs				"accept"
 
@@ -762,8 +803,8 @@ do {														\
 #define MSPROXY_CONNECT				0x071e	/* connect request.						*/
 #define MSPROXY_CONNECT_ACK		0x0703	/* connect request accepted.			*/
 
-#define MSPROXY_UDPASSOCIATE		0x0705	/* udp associate request.				*/
-#define MSPROXY_UDPASSOCIATE_ACK	0x0706	/* udp associate request accepted.	*/
+#define MSPROXY_UDPASSOCIATE		0x0705	/* UDP associate request.				*/
+#define MSPROXY_UDPASSOCIATE_ACK	0x0706	/* UDP associate request accepted.	*/
 
 #define MSPROXY_CONNECTED			0x042c	/* client connected to server?		*/
 
@@ -784,15 +825,6 @@ do {														\
 #define SOCKS_RECV		0
 #define SOCKS_SEND		1
 
-#if 0
-/* where is this from? (michaels) */
-#if defined(__alpha__)
-typedef unsigned int u_int32;
-#else
-typedef unsigned long u_int32;
-#endif
-#endif
-
 /* offsets into authentication packet */
 #define AUTH_VERSION		0	/* version of method packet.								*/
 
@@ -808,25 +840,9 @@ typedef unsigned long u_int32;
 #define UNAME_VERSION	0
 #define UNAME_STATUS		1
 
-/* XXX no ivp6 support currently. */
+/* XXX no IPv6 support currently. */
 #define SOCKS_IPV6_ALEN 16
 
-#if !defined(SOCKSLIBRARY_DYNAMIC)
-#define sys_accept(s, addr, addrlen)		accept(s, addr, addrlen)
-#define sys_bind(s, name, namelen)			bind(s, name, namelen)
-#define sys_connect(s, name, namelen)		connect(s, name, namelen)
-#define sys_bindresvport(sd, sin)			bindresvport(sd, sin)
-#define sys_gethostbyname(name)				gethostbyname(name)
-#define sys_gethostbyname2(name, af)		gethostbyname2(name, af)
-#define sys_getpeername(s, name, namelen)	getpeername(s, name, namelen)
-#define sys_getsockname(s, name, namelen)	getsockname(s, name, namelen)
-#define sys_recvfrom(s, buf, len, flags, from, fromlen) \
-		  recvfrom(s, buf, len, flags, from, fromlen)
-#define sys_rresvport(port)					rresvport(port)
-#define sys_sendto(s, msg, len, flags, to, tolen) \
-		  sendto(s, msg, len, flags, to, tolen)
-#endif
-
 enum operator_t { none = 0, eq, neq, ge, le, gt, lt, range };
 
 struct compat_t {
@@ -841,7 +857,7 @@ struct logtype_t {
 	FILE				**fpv;		/* if logging is to file, this is the open file.*/
 	int				fpc;
 	int				*fplockv;	/* locking of logfiles.									*/
-	int				facility;	/* if logging to syslog, this is the facility.						*/
+	int				facility;	/* if logging to syslog, this is the facility.	*/
 };
 
 
@@ -1151,7 +1167,7 @@ struct response_t {
 	struct authmethod_t	*auth;	/* pointer to level above. */
 };
 
-/* encapsulation for udp packets. */
+/* encapsulation for UDP packets. */
 struct udpheader_t {
 	unsigned char			flag[2];
 	unsigned char			frag;
@@ -1159,25 +1175,16 @@ struct udpheader_t {
 };
 
 
-/* interface request packet. */
-struct interfacerequest_t {
-	unsigned char			rsv;
-	unsigned char			sub;
-	unsigned char			flag;
-	struct sockshost_t	host;
-};
-
-
 /* method username */
 struct uname_t {
 	unsigned char	version;
-	char				name[MAXNAMELEN];
-	char				password[MAXPWLEN];
+	unsigned char	name[MAXNAMELEN];
+	unsigned char	password[MAXPWLEN];
 };
 
 /* method rfc931 */
 struct rfc931_t {
-	char				name[MAXNAMELEN];
+	unsigned char	name[MAXNAMELEN];
 };
 
 /* this must be big enough to hold a complete method request. */
@@ -1212,7 +1219,7 @@ struct command_t {
 
 	/* not real commands as per standard, but they have their use. */
 	unsigned bindreply:1;		/* reply to bind command.	*/
-	unsigned udpreply:1;			/* reply to udp packet.		*/
+	unsigned udpreply:1;			/* reply to UDP packet.		*/
 	unsigned :0;
 };
 
@@ -1236,7 +1243,7 @@ struct serverstate_t {
 
 
 struct msproxy_state_t {
-	struct sockaddr_in		controladdr;	/* udp address of proxyserver.		*/
+	struct sockaddr_in		controladdr;	/* UDP address of proxyserver.		*/
 	int32_t						magic25;
 	int32_t						bindid;
 	int32_t						clientid;
@@ -1263,9 +1270,9 @@ struct socksstate_t {
 #endif
 	struct msproxy_state_t	msproxy;			/* if msproxy, msproxy state.			*/
 	struct protocol_t			protocol;		/* protocol in use.						*/
-	unsigned						udpconnect:1;	/* connected udp socket?				*/
+	unsigned						udpconnect:1;	/* connected UDP socket?				*/
 	int							system;			/* don't check, use system call.		*/
-	int							version;			/* version									*/
+	int							version;			/* version (-1)							*/
 };
 
 
@@ -1283,8 +1290,8 @@ struct ruleaddress_t {
 	} addr;
 
 	struct {
-		in_port_t			tcp;			/* tcp portstart or field to operator on.	*/
-		in_port_t			udp;			/* udp portstart or field to operator on.	*/
+		in_port_t			tcp;			/* TCP portstart or field to operator on.	*/
+		in_port_t			udp;			/* UDP portstart or field to operator on.	*/
 	} port;
 	in_port_t				portend;		/* only used if operator is range.			*/
 	enum operator_t		operator;	/* operator to compare ports via.			*/
@@ -1324,10 +1331,7 @@ struct socks_t {
 	struct socksstate_t			state;
 };
 
-
-
 enum portcmp { e_lt, e_gt, e_eq, e_neq, e_le, e_ge, e_nil };
-typedef enum portcmp Portcmp;
 
 
 
@@ -1397,9 +1401,11 @@ genericinit __P((void));
  */
 
 void
-initlog __P((void));
+newprocinit __P((void));
 /*
- * (Re)initialize logging.
+ * After a new process is started/forked, this function should
+ * be called.  It will initialize various things, open needed
+ * descriptors, etc. and can be called as many times as wanted.
  */
 
 
@@ -1483,7 +1489,7 @@ socks_logmatch __P((unsigned int d, const struct logtype_t *log));
 char *
 sockaddr2string __P((const struct sockaddr *address, char *string, size_t len));
 /*
- * Returns the ip address and port in "address" on string form.
+ * Returns the IP address and port in "address" on string form.
  * "address" is assumed to be on network form and it will be
  * converted to host form before written to "string".
  * "len" gives length of the NUL terminated string.
@@ -1559,17 +1565,35 @@ recvmsgn __P((int s, struct msghdr *msg, int flags, size_t len));
  */
 
 ssize_t
-readn __P((int, void *, size_t));
+readn __P((int, void *, size_t, struct authmethod_t *auth));
 /*
- * Like read() but retries.
+ * Like read() but retries and takes an additional "auth" argument
+ * to be used if not NULL.
  */
 
 ssize_t
-writen __P((int, const void *, size_t));
+writen __P((int, const void *, size_t, struct authmethod_t *auth));
 /*
- * like write() but retries.
+ * like write() but retries and takes an additional "auth" argument
+ * to be used if not NULL.
  */
 
+ssize_t
+socks_recvfrom __P((int, void *, size_t, int, struct sockaddr *, socklen_t *,
+						  struct authmethod_t *auth));
+/*
+ * Like recvfrom(), but with an additional auth argument to be used
+ * if not NULL.
+*/
+
+ssize_t
+socks_sendto __P((int, const void *, size_t, int, const struct sockaddr *,
+						socklen_t, struct authmethod_t *auth));
+/*
+ * Like sendto(), but with an additional auth argument to be used
+ * if not NULL.
+*/
+
 int
 closen __P((int));
 /*
@@ -1637,8 +1661,9 @@ string2method __P((const char *methodname));
  */
 
 
-char *
-sockshost2mem __P((const struct sockshost_t *host, char *mem, int version));
+unsigned char *
+sockshost2mem __P((const struct sockshost_t *host, unsigned char *mem,
+						 int version));
 /*
  * Writes "host" out to "mem".  The caller must make sure "mem"
  * is big enough to hold the contents of "host".
@@ -1646,9 +1671,9 @@ sockshost2mem __P((const struct sockshost_t *host, char *mem, int version));
  * Returns a pointer to one element past the last byte written to "mem".
  */
 
-const char *
-mem2sockshost __P((struct sockshost_t *host, const char *mem, size_t len,
-						 int version));
+const unsigned char *
+mem2sockshost __P((struct sockshost_t *host, const unsigned char *mem,
+						 size_t len, int version));
 /*
  * Writes "mem", which is assumed to be a sockshost string
  * of version "version" in network order, out to "host".
@@ -1684,6 +1709,12 @@ readconfig __P((const char *filename));
  *		On failure: -1.
  */
 
+void
+yywarn __P((const char *s));
+/*
+ * Report a error related to (configfile) parsing.
+ */
+
 void
 yyerror __P((const char *s));
 /*
@@ -1698,7 +1729,7 @@ addressmatch __P((const struct ruleaddress_t *rule,
  * Tries to match "address" against "rule".  "address" is resolved
  * if necessary.  "address" supports the wildcard INADDR_ANY and port of 0.
  * "protocol" is the protocol to compare under.
- * If "ipalias" is true, the function will try to match any ip alias
+ * If "ipalias" is true, the function will try to match any IP alias
  * "address"'s might have if appropriate, this can be useful to match
  * multihomed hosts where the client requests e.g a bind connection.
  * Returns true if "rule" matched "address".
@@ -1725,7 +1756,7 @@ hostentfree __P((struct hostent *hostent));
 int
 socks_connect __P((int s, const struct sockshost_t *host));
 /*
- * Tries to connect to "host".  If "host"'s address is not a ip address,
+ * Tries to connect to "host".  If "host"'s address is not a IP address,
  * the function also tries to connect to any alias for "host"'s
  * name.  The connection is done using the open descriptor "s".
  * Returns:
@@ -1773,6 +1804,11 @@ showstate __P((const struct serverstate_t *state));
  * Shows "state".
  */
 
+void
+showmethod __P((int methodc, const int *methodv));
+/*
+ * Shows "methodv".
+*/
 
 struct route_t *
 addroute __P((const struct route_t *route));
@@ -1815,14 +1851,14 @@ ruleaddress2string __P((const struct ruleaddress_t *rule, char *string,
  */
 
 
-int
+unsigned char
 sockscode __P((int version, int code));
 /*
  * Maps the socks replycode "code", which is in non-version specific format,
  * to the equivalent replycode in version "version".
  */
 
-int
+unsigned char
 errno2reply __P((int errnum, int version));
 /*
  * Returns the socks version "version" reply code for a error of type "errno".
@@ -1854,7 +1890,7 @@ str2vis __P((const char *string, size_t len));
 in_addr_t
 socks_addfakeip __P((const char *name));
 /*
- * Adds "name" to a internal table indexed by (fake)ip addresses.
+ * Adds "name" to a internal table indexed by (fake)IP addresses.
  * Returns:
  *		On success: "name"'s index.
  *		On failure:	INADDR_NONE
@@ -1966,6 +2002,11 @@ socks_unlock __P((int d));
  * Unlocks the filedescriptor "d", previously locked by this process.
  */
 
+#if SOCKSLIBRARY_DYNAMIC
+struct hostent *sys_gethostbyaddr __P((const char *addr, int len, int af));
+struct hostent *sys_gethostbyname __P((const char *));
+struct hostent *sys_gethostbyname2 __P((const char *, int));
+#endif /* SOCKSLIBRARY_DYNAMIC */
 
 #if defined(DEBUG) || HAVE_SOLARIS_BUGS
 
@@ -2050,8 +2091,19 @@ int issetugid __P((void));
 void vsyslog __P((int, const char *, va_list));
 #endif  /* !HAVE_VSYSLOG */
 
+struct passwd *
+socks_getpwnam __P((const char *login));
+/*
+ * Like getpwnam() but works around sysv bug, tries to get the shadow
+ * password too.
+*/
+
 __END_DECLS
 
+#if SOCKSLIBRARY_DYNAMIC
+#include "interposition.h"
+#endif /* SOCKSLIBRARY_DYNAMIC */
+
 /* XXX */
 #if defined(SOCKS_CLIENT) || defined(SOCKS_SERVER)
 #if SOCKS_CLIENT
