@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -41,22 +41,24 @@
  *
  */
 
-/* $Id: sockd.h,v 1.136 1999/12/22 09:29:20 karls Exp $ */
+/* $Id: sockd.h,v 1.150 2000/06/21 09:38:22 michaels Exp $ */
 
 #ifndef _SOCKD_H_
 #define _SOCKD_H_
 #endif
 
-/* use caching version in server. */
-#define gethostbyname(name)				cgethostbyname(name)
-#define gethostbyaddr(addr, len, type)	cgethostbyaddr(addr, len, type)
-
 #ifdef lint
 extern const int lintnoloop_sockd_h;
 #else
 #define lintnoloop_sockd_h 0
 #endif
 
+/* use caching versions directly, avoid overhead. */
+#undef gethostbyname
+#define gethostbyname(name)				cgethostbyname(name)
+#undef gethostbyaddr
+#define gethostbyaddr(addr, len, type)	cgethostbyaddr(addr, len, type)
+
 
 #define INIT(length)									\
 	const size_t start	= state->start;		\
@@ -70,7 +72,8 @@ extern const int lintnoloop_sockd_h;
  * Returns the number of bytes left to read.
  */
 
-#define READ(s, length)	(readn((s), &state->mem[state->reqread], (length)))
+#define READ(s, length, auth)	\
+	(readn((s), &state->mem[state->reqread], (length), auth))
 /*
  * "s" is the descriptor to read from.
  * "length" is how much to read.
@@ -84,7 +87,7 @@ extern const int lintnoloop_sockd_h;
  * Fills "object" with data.
  */
 
-#define CHECK(object, nextfunction)									\
+#define CHECK(object, auth, nextfunction)							\
 do {																			\
 	int p;																	\
 																				\
@@ -96,7 +99,7 @@ do {																			\
 			SERRX(MEMLEFT());												\
 																				\
 		errno = 0;															\
-		if ((p = READ(s, LEFT())) <= 0)								\
+		if ((p = READ(s, LEFT(), auth)) <= 0)						\
 			return p;														\
 		state->reqread += p;												\
 																				\
@@ -111,7 +114,12 @@ do {																			\
 			return state->rcurrent(s, request, state);			\
 	}																			\
 } while (lintnoloop_sockd_h)
-
+/*
+ * Checks whether "object" has been filled with all data requested and
+ * if so calls "function", if function is not NULL.
+ * If "object" has not been filled it returns the number of bytes
+ * that was added to object on this call, or error.
+*/
 
 
 #define SOCKD_NEWREQUEST	1	/* sending a new request	*/
@@ -140,7 +148,7 @@ do {																			\
 
 	/*
 	 * config stuff
-	*/
+	 */
 
 #define VERDICT_BLOCKs		"block"
 #define VERDICT_PASSs		"pass"
@@ -156,8 +164,7 @@ do {																			\
 #define OPERATION_ACCEPT		1
 #define OPERATION_CONNECT		(OPERATION_ACCEPT + 1)
 #define OPERATION_IO				(OPERATION_CONNECT + 1)
-#define OPERATION_DISCONNECT	(OPERATION_IO + 1)
-#define OPERATION_ABORT			(OPERATION_DISCONNECT + 1)
+#define OPERATION_ABORT			(OPERATION_IO + 1)
 #define OPERATION_ERROR			(OPERATION_ABORT + 1)
 
 
@@ -185,10 +192,10 @@ struct linkedname_t {
 
 /* linked list over current rules. */
 struct rule_t {
+	struct ruleaddress_t		src;				/* src.										*/
 	struct ruleaddress_t		dst;				/* dst.										*/
 	struct log_t				log;				/* type of logging to do.				*/
 	int							number;			/* rulenumber, info/debugging only.	*/
-	struct ruleaddress_t		src;				/* src.										*/
 	struct serverstate_t		state;
 	struct linkedname_t		*user;			/* name of users allowed.				*/
 	int							verdict;			/* verdict for this rule.				*/
@@ -228,7 +235,11 @@ struct userid_t {
 
 struct configstate_t {
 	unsigned						init:1;
-	volatile sig_atomic_t	addchild;				/* okay to do a addchild()?	*/
+#ifdef HAVE_VOLATILE_SIG_ATOMIC_T
+	sig_atomic_t				addchild;				/* okay to do a addchild()?	*/
+#else
+	volatile sig_atomic_t   addchild;            /* okay to do a addchild()?   */
+#endif
 	uid_t							euid;						/* original euid.					*/
 	pid_t							*motherpidv;			/* pid of mothers.				*/
 	pid_t							pid;						/* pid of current process.		*/
@@ -294,7 +305,7 @@ struct config_t {
 
 
 struct connectionstate_t {
-	struct authmethod_t	auth;
+	struct authmethod_t	auth;					/* XXX should probably not be here. */
 	int						command;
 	struct extension_t	extension;			/* extensions set.						*/
 	int						protocol;
@@ -310,7 +321,16 @@ struct sockd_io_direction_t {
 	int								s;				/* socket connection.					*/
 	struct sockaddr_in			laddr;		/* local address of s.					*/
 	struct sockaddr_in			raddr;		/* address of remote peer for s.		*/
-	struct connectionstate_t	state;
+
+	/*
+	 * Varies according to context.
+	 * src:		as laddr but on sockshost_t form.
+	 * dst:		name as given by client.
+	 * control: as laddr
+	*/
+	struct sockshost_t			host;
+
+	struct authmethod_t			auth;			/* authentication used.					*/
 
 	size_t							sndlowat;	/* low-water mark for send.			*/
 
@@ -327,16 +347,13 @@ struct sockd_io_t {
 
 	struct connectionstate_t		state;
 
-	struct sockd_io_direction_t	in;			/* client we receive data from.	*/
-	struct sockd_io_direction_t	out;			/* remote peer.						*/
 	struct sockd_io_direction_t	control;		/* control connection to client.	*/
+	struct sockd_io_direction_t	src;			/* client we receive data from.	*/
+	struct sockd_io_direction_t	dst;			/* remote peer.						*/
 
 	struct rule_t						acceptrule;	/* rule matched for accept().		*/
 	struct rule_t						rule;			/* matched rule for i/o.			*/
 
-	struct sockshost_t				src;			/* our client.							*/
-	struct sockshost_t				dst;			/* it's desired peer.				*/
-
 	time_t								time;			/* time of last i/o operation.	*/
 	struct sockd_io_t					*next;		/* for some special cases.			*/
 };
@@ -533,8 +550,8 @@ rulespermit __P((int s, struct rule_t *rule,
 					  const struct sockshost_t *src, const struct sockshost_t *dst));
 /*
  * Checks whether the rules permit data from "src" to "dst".
- * "s" is the socket the connection is on.  "state" is the current
- * state of the connection and may be updated.
+ * "s" is the socket the connection is on.
+ * "state" is the current state of the connection and may be updated.
  *
  * Wildcard fields are supported for the following fields;
  *		ipv4:			INADDR_ANY
@@ -637,7 +654,8 @@ method_uname __P((int s, struct request_t *request,
 void
 iolog __P((struct rule_t *rule, const struct connectionstate_t *state,
 		int operation,
-		const struct sockshost_t *src, const struct sockshost_t *dst,
+		const struct sockshost_t *src, const struct authmethod_t *srcauth,
+		const struct sockshost_t *dst, const struct authmethod_t *dstauth,
 		const char *data, size_t count));
 /*
  * Called after each each complete io operation
@@ -654,7 +672,6 @@ iolog __P((struct rule_t *rule, const struct connectionstate_t *state,
  * If "operation" is
  *    OPERATION_ACCEPT
  *		OPERATION_CONNECT
- *    OPERATION_DISCONNECT
  *			"data" and "count" is ignored.
  *
  *		OPERATION_ABORT
@@ -898,7 +915,6 @@ passwordcheck __P((const char *name, const char *cleartextpassword));
  *		If "cleartextpassword" does not match: 2
  */
 
-
 #ifdef DEBUG
 void
 printfd __P((const struct sockd_io_t *io, const char *prefix));
