@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: Rbind.c,v 1.102 1999/12/22 09:29:21 karls Exp $";
+"$Id: Rbind.c,v 1.103 2000/05/31 12:14:51 karls Exp $";
 
 int
 Rbind(s, name, namelen)
@@ -75,7 +75,7 @@ Rbind(s, name, namelen)
 				/*
 				 * We try to make the client think it's address is the address
 				 * the server is using on it's behalf.  Some clients might try
-				 * bind that ip address (with a different port, presumably)
+				 * bind that IP address (with a different port, presumably)
 				 * themselves though, in that case, use INADDR_ANY.
 				 */
 
@@ -243,7 +243,7 @@ Rbind(s, name, namelen)
 		case SOCKS_V4:
 			/* LINTED pointer casts may be troublesome */
 			if (((struct sockaddr_in *)&socksfd.remote)->sin_addr.s_addr
-			== htonl(0)) { /* v4 spesific; remote doesn't know, set to remote. */
+			== htonl(0)) { /* v4 specific; remote doesn't know, set to remote. */
 				struct sockaddr_in addr;
 
 				len = sizeof(addr);
@@ -258,7 +258,7 @@ Rbind(s, name, namelen)
 			/* FALLTHROUGH */
 
 		case SOCKS_V5:
-			socksfd.reply						= socksfd.remote;	/* same ip address. */
+			socksfd.reply						= socksfd.remote;	/* same IP address. */
 			socksfd.state.acceptpending	= socksfd.route->gw.state.extension.bind;
 			break;
 
