@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: connectchild.c,v 1.91 1999/10/04 12:43:37 michaels Exp $";
+"$Id: connectchild.c,v 1.96 2000/06/08 12:47:56 michaels Exp $";
 
 #define MOTHER 0	/* descriptor mother reads/writes on.  */
 #define CHILD	1	/* descriptor child reads/writes on.   */
@@ -156,21 +156,19 @@ socks_nbconnectroute(s, control, packet, src, dst)
 				struct itimerval timerval;
 				size_t i, max;
 
-				config.state.pid = getpid();
-
 				slog(LOG_DEBUG, "%s: connectchild forked", function);
 
-				setsid();
-
 				/* close unknown descriptors. */
 				for (i = 0, max = getdtablesize(); i < max; ++i)
 					if (socks_logmatch(i, &config.log)
 					|| i == (unsigned int)pipev[CHILD])
 						continue;
+					else if (isatty(i))
+						continue;
 					else
 						close((int)i);
 
-				initlog();
+				newprocinit();
 
 				/*
 				 * in case of using msproxy stuff, don't want mothers mess,
@@ -266,7 +264,7 @@ socks_nbconnectroute(s, control, packet, src, dst)
 	if (!ADDRISBOUND(local)) {
 		bzero(&local, sizeof(local));
 
-		/* bind same ip as control, any fixed address would do though. */
+		/* bind same IP as control, any fixed address would do though. */
 
 		len = sizeof(local);
 		/* LINTED pointer casts may be troublesome */
