@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -45,10 +45,17 @@
 #include "config_parse.h"
 
 static const char rcsid[] =
-"$Id: serverconfig.c,v 1.92 1999/12/22 09:29:25 karls Exp $";
+"$Id: serverconfig.c,v 1.113 2000/06/26 10:43:21 karls Exp $";
 
 __BEGIN_DECLS
 
+static const char *
+authinfo __P((const struct authmethod_t *auth, char *info, size_t infolen));
+/*
+ * Fills in "info" with a printable representation of the "auth".
+ * Returns a pointer to "info".
+*/
+
 static void
 showuser __P((const struct linkedname_t *user));
 /*
@@ -71,17 +78,14 @@ libwrapinit __P((int s, struct request_info *request));
  */
 
 static int
-connectisok __P((struct request_info *request, const struct rule_t *rule,
-					  struct connectionstate_t *state));
+connectisok __P((struct request_info *request, const struct rule_t *rule));
 #else /* !HAVE_LIBWRAP */
 static int
-connectisok __P((void *request, const struct rule_t *rule,
-					  struct connectionstate_t *state));
+connectisok __P((void *request, const struct rule_t *rule));
 #endif /* !HAVE_LIBWRAP */
 /*
  * Checks the connection on "s".
- * "rule" is the rule that matched the connection and "state" is the current
- * state.
+ * "rule" is the rule that matched the connection.
  * This function should be called after each rulecheck for a new
  * connection/packet.
  *
@@ -91,9 +95,11 @@ connectisok __P((void *request, const struct rule_t *rule,
  */
 
 static struct rule_t *
-addrule __P((const struct rule_t *newrule, struct rule_t **rulebase));
+addrule __P((const struct rule_t *newrule, struct rule_t **rulebase,
+				 int client));
 /*
  * Appends a copy of "newrule" to "rulebase".
+ * If "client" is true, "newrule" is a clientrule.
  * Returns a pointer to the added rule (not "newrule").
  */
 
@@ -116,30 +122,43 @@ struct rule_t *
 addclientrule(newrule)
 	const struct rule_t *newrule;
 {
-	struct rule_t *rule;
+	struct rule_t *rule, ruletoadd;
+	int i;
 
-	rule = addrule(newrule, &config.crule);
+	ruletoadd = *newrule; /* for const. */
 
-	/*
-	 * there are a few things that need to be changed versus the generic
-	 * init done by addrule().
-	 */
+	rule = addrule(&ruletoadd, &config.crule, 1);
+
+	for (i = 0; i < rule->state.methodc; ++i)
+		switch (rule->state.methodv[i]) {
+			case AUTHMETHOD_NONE:
+			case AUTHMETHOD_RFC931:
+				break;
+
+			default: {
+				char buf[256];
+
+				snprintf(buf, sizeof(buf), "method %s is not valid for clientrules",
+				method2string(rule->state.methodv[i]));
+
+				yyerror(buf);
+			}
+		}
 
 	if (rule->user != NULL) {
-		/*
-		 * this is a clientrule so strip away any methods that
-		 * can not provide a username without socks negotiation.
-		 */
-		int i;
-
+		/* check that all methods given handle usernames. */
 		for (i = 0; i < rule->state.methodc; ++i)
 			switch (rule->state.methodv[i]) {
 				case AUTHMETHOD_RFC931:
 					break;
 
-				default:
-					rule->state.methodv[i--]
-					= rule->state.methodv[--rule->state.methodc];
+				default: {
+					char buf[256];
+
+					snprintf(buf, sizeof(buf), "method \"%s\" can not provide "
+					"usernames", method2string(rule->state.methodv[i]));
+					yyerror(buf);
+				}
 			}
 	}
 
@@ -156,25 +175,7 @@ addsocksrule(newrule)
 
 	struct rule_t *rule;
 
-	rule = addrule(newrule, &config.srule);
-
-	if (rule->user == NULL) {
-		/*
-		 * For each method taking a username, default to
-		 * allowing everybody that's in the passwordfile.
-		 */
-		int i;
-
-		for (i = 0; i < rule->state.methodc; ++i)
-			switch (rule->state.methodv[i]) {
-				case AUTHMETHOD_UNAME:
-				case AUTHMETHOD_RFC931:
-					if (adduser(&rule->user, method2string(rule->state.methodv[i]))
-					== NULL)
-						serrx(EXIT_FAILURE, NOMEM);
-					break;
-			}
-	}
+	rule = addrule(newrule, &config.srule, 0);
 
 	checkrule(rule);
 
@@ -249,12 +250,14 @@ showclient(rule)
 	slog(LOG_INFO, "verdict: %s",
 	rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs);
 
-	slog(LOG_INFO, "from: %s",
+	slog(LOG_INFO, "src: %s",
 	ruleaddress2string(&rule->src, addr, sizeof(addr)));
 
-	slog(LOG_INFO, "to: %s",
+	slog(LOG_INFO, "dst: %s",
 	ruleaddress2string(&rule->dst, addr, sizeof(addr)));
 
+	showmethod(rule->state.methodc, rule->state.methodv);
+
 	showuser(rule->user);
 
 	showlog(&rule->log);
@@ -459,7 +462,7 @@ resetconfig(void)
 	bzero(&config.uid, sizeof(config.uid));
 
 	/*
-	 * initialize misc options to sensible default.
+	 * initialize misc. options to sensible default.
 	 */
 
 	config.resolveprotocol		= RESOLVEPROTOCOL_UDP;
@@ -469,76 +472,55 @@ resetconfig(void)
 }
 
 void
-iolog(rule, state, operation, src, dst, data, count)
+iolog(rule, state, operation, src, srcauth, dst, dstauth, data, count)
 	struct rule_t *rule;
 	const struct connectionstate_t *state;
 	int operation;
-	const struct sockshost_t *src, *dst;
+	const struct sockshost_t *src;
+	const struct authmethod_t *srcauth;
+	const struct sockshost_t *dst;
+	const struct authmethod_t *dstauth;
 	const char *data;
 	size_t count;
 {
-	char srcstring[MAXSOCKSHOSTSTRING + MAXNAMELEN + sizeof("@") - 1];
-	char dststring[MAXSOCKSHOSTSTRING];
-	const char *name;
+	/* CONSTCOND */
+	char srcstring[MAXSOCKSHOSTSTRING + MAXAUTHINFOLEN];
+	char dststring[sizeof(srcstring)];
+	char rulecommand[256];
 	int p;
 
-	name = NULL;
-	switch (state->auth.method) {
-		case AUTHMETHOD_NONE:
-		case AUTHMETHOD_NOACCEPT: /* closing connection. */
-			/*
-			 * doesn't take any space so it's possible it has a name, even
-			 * if method doesn't indicate it.
-			 */
-			name = state->auth.mdata.rfc931.name;
-			break;
-
-		case AUTHMETHOD_UNAME:
-			name = state->auth.mdata.uname.name;
-			break;
-
-		case AUTHMETHOD_RFC931:
-			name = state->auth.mdata.rfc931.name;
-			break;
-
-		default:
-			SERRX(state->auth.method);
-	}
-
-	if (name != NULL && *name != NUL)
-		p = snprintf(srcstring, sizeof(srcstring), "%s@", name);
-	else
-		p = 0;
-
+	authinfo(srcauth, srcstring, sizeof(srcstring));
+	p = strlen(srcstring);
 	sockshost2string(src, &srcstring[p], sizeof(srcstring) - p);
-	sockshost2string(dst, dststring, sizeof(dststring));
 
-	/* XXX should probably include authinfo somewhere here too. */
+	authinfo(dstauth, dststring, sizeof(dststring));
+	p = strlen(dststring);
+	sockshost2string(dst, &dststring[p], sizeof(dststring) - p);
+
+	snprintf(rulecommand, sizeof(rulecommand), "%s(%d): %s/%s", 
+	rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs,
+	rule->number, protocol2string(state->protocol),
+	command2string(state->command));
+
 	switch (operation) {
 		case OPERATION_ACCEPT:
-		case OPERATION_DISCONNECT:
 		case OPERATION_CONNECT:
-			if (rule->log.connect || rule->log.disconnect)
-				slog(LOG_INFO, "%s(%d): %s: %s -> %s",
-				rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs,
-				rule->number, command2string(state->command),
-				srcstring, dststring);
+			if (rule->log.connect)
+				slog(LOG_INFO, "%s [: %s -> %s", rulecommand, srcstring, dststring);
 			break;
 
 		case OPERATION_ABORT:
 			if (rule->log.disconnect || rule->log.error)
-				slog(LOG_INFO, "%s(%d): %s abort: %s -> %s: %s",
-				rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs,
-				rule->number, command2string(state->command), srcstring, dststring,
+				slog(LOG_INFO, "%s ]: %s -> %s: %s",
+				rulecommand, srcstring, dststring,
 				data == NULL ? strerror(errno) : data);
 			break;
 
 		case OPERATION_ERROR:
 			if (rule->log.error)
-				slog(LOG_INFO, "%s(%d): %s error: %s -> %s: %s",
-				rule->verdict == VERDICT_PASS ? VERDICT_PASSs : VERDICT_BLOCKs,
-				rule->number, protocol2string(state->protocol), srcstring,
-				dststring, data == NULL ? strerror(errno) : data);
+				slog(LOG_INFO, "%s ]: %s -> %s: %s",
+				rulecommand, srcstring, dststring,
+				data == NULL ? strerror(errno) : data);
 			break;
 
 		case OPERATION_IO:
@@ -547,19 +529,17 @@ iolog(rule, state, operation, src, dst, data, count)
 
 				SASSERTX(data != NULL);
 
-				slog(LOG_INFO, "%s(%d): %s: %s -> %s (%lu): %s",
-				rule->verdict == VERDICT_BLOCK ? VERDICT_BLOCKs : VERDICT_PASSs,
-				rule->number, protocol2string(state->protocol),
-				srcstring, dststring, (unsigned long)count,
+				slog(LOG_INFO, "%s -: %s -> %s (%lu): %s",
+				rulecommand, srcstring, dststring, (unsigned long)count,
 				strcheck(visdata = str2vis(data, count)));
 
 				free(visdata);
+				break;
 			}
-			else if (rule->log.iooperation)
-				slog(LOG_INFO, "%s(%d): %s: %s -> %s (%lu)",
-				rule->verdict == VERDICT_BLOCK ? VERDICT_BLOCKs : VERDICT_PASSs,
-				rule->number, protocol2string(state->protocol),
-				srcstring, dststring, (unsigned long)count);
+
+			if (rule->log.iooperation)
+				slog(LOG_INFO, "%s -: %s -> %s (%lu)",
+				rulecommand, srcstring, dststring, (unsigned long)count);
 			break;
 
 		default:
@@ -603,15 +583,13 @@ rulespermit(s, match, state, src, dst)
 
 		defrule.dst									= defrule.src;
 
+		memset(&defrule.log, 0, sizeof(defrule.log));
+		defrule.log.connect		= 1;
+		defrule.log.iooperation	= 1; /* blocked iooperations. */
+
 		if (config.option.debug) {
-			defrule.log.connect		= 1;
 			defrule.log.disconnect	= 1;
 			defrule.log.error			= 1;
-			defrule.log.iooperation	= 1;
-		}
-		else {
-			memset(&defrule.log, 0, sizeof(defrule.log));
-			defrule.log.connect = 1;
 		}
 
 		memset(&defrule.state.command, UCHAR_MAX, sizeof(defrule.state.command));
@@ -730,9 +708,15 @@ rulespermit(s, match, state, src, dst)
 #if HAVE_LIBWRAP
 			if (methodisset(AUTHMETHOD_RFC931, rule->state.methodv,
 			(size_t)rule->state.methodc)) {
-				strncpy(state->auth.mdata.rfc931.name, eval_user(&libwraprequest),
+				strncpy((char *)state->auth.mdata.rfc931.name,
+				eval_user(&libwraprequest),
 				sizeof(state->auth.mdata.rfc931.name) - 1);
 
+				/* libwrap sets it to unknown if no identreply. */
+				if (strcmp((char *)state->auth.mdata.rfc931.name, STRING_UNKNOWN)
+				== 0)
+					*state->auth.mdata.rfc931.name = NUL;
+
 				if (state->auth.mdata.rfc931.name[
 				sizeof(state->auth.mdata.rfc931.name) - 1] != NUL) {
 					slog(LOG_INFO, "%s: rfc931 name truncated", function);
@@ -740,7 +724,10 @@ rulespermit(s, match, state, src, dst)
 					sizeof(state->auth.mdata.rfc931.name) - 1] = NUL;
 				}
 
-				state->auth.method = AUTHMETHOD_RFC931;
+				if (*state->auth.mdata.rfc931.name != NUL)
+					state->auth.method = AUTHMETHOD_RFC931;
+				else
+					continue;
 			}
 			else
 #endif /* HAVE_LIBWRAP */
@@ -751,54 +738,44 @@ rulespermit(s, match, state, src, dst)
 
 		switch (state->auth.method) {
 			case AUTHMETHOD_UNAME:
-				name		= state->auth.mdata.uname.name;
-				password	= state->auth.mdata.uname.password;
+				name		= (char *)state->auth.mdata.uname.name;
+				password	= (char *)state->auth.mdata.uname.password;
 				break;
 
 			case AUTHMETHOD_RFC931:
-				name		= state->auth.mdata.rfc931.name;
+				name		= (char *)state->auth.mdata.rfc931.name;
 				password	= NULL;
 				break;
 
 			default:
+				/* not usernamebased. */
 				name = password = NULL;
 		}
 
 		if (name != NULL && rule->user != NULL) {
-			const char *methodname;
-			char srcstring[MAXSOCKSHOSTSTRING];
 			struct linkedname_t *ruleuser;
 
 			/*
 			 * The rule->user names restricts access further, only names
-			 * appearing there and in the passwordfile are matched.
-			 * An alias for "everyone" is a name that is the same as the
-			 * name of the selected method.
+			 * appearing there are checked.
 			 */
-
-			methodname = method2string(state->auth.method);
 			ruleuser = rule->user;
-			do {
-				if (strcmp(methodname, ruleuser->name) == 0)
-					break; /* all usernames "match" here. */
-				else if (string2method(name) >= 0)
-					slog(LOG_INFO, "%s: suspicious username from %s: %s",
-					function, sockshost2string(src, srcstring, sizeof(srcstring)),
-					name);
-				else if (strcmp(name, ruleuser->name) == 0)
+			do
+				if (strcmp(name, ruleuser->name) == 0)
 					break;
-			} while ((ruleuser = ruleuser->next) != NULL);
+			while ((ruleuser = ruleuser->next) != NULL);
 
 			if (ruleuser == NULL)
 				continue; /* no match. */
+		}
 
+		if (name != NULL)
 			if (!state->auth.matched) { /* may have been checked at lower level. */
-				/* all users must also be in passwordfile. */
+				/* name must also be in passwordfile. */
 				if (passwordcheck(name, password) != 0)
 					continue;
 				state->auth.matched = 1;
 			}
-		}
 
 		/*
 		 * This is a little tricky, but for some commands we may not
@@ -839,7 +816,7 @@ rulespermit(s, match, state, src, dst)
 	 * requires the rule matched so needs to be delayed til here.
 	 */
 
-	if (!connectisok(&libwraprequest, match, state))
+	if (!connectisok(&libwraprequest, match))
 		match->verdict = VERDICT_BLOCK;
 
 	/*
@@ -860,17 +837,60 @@ rulespermit(s, match, state, src, dst)
 	return match->verdict == VERDICT_PASS;
 }
 
+static const char *
+authinfo(auth, info, infolen)
+	const struct authmethod_t *auth;
+	char *info;
+	size_t infolen;
+{
+	const char *authname, *methodname;
+
+	if (auth != NULL) {
+		methodname = method2string(auth->method);
+
+		switch (auth->method) {
+			case AUTHMETHOD_NONE:
+			case AUTHMETHOD_NOACCEPT: /* closing connection next presumably. */
+				authname = methodname = NULL;
+				break;
+
+			case AUTHMETHOD_UNAME:
+				authname = (const char *)auth->mdata.uname.name;
+				break;
+
+			case AUTHMETHOD_RFC931:
+				authname = (const char *)auth->mdata.rfc931.name;
+				break;
+
+			default:
+				SERRX(auth->method);
+		}
+	}
+	else
+		authname = methodname = NULL;
+
+	if (authname == NULL || *authname == NUL)
+		*info = NUL;
+	else
+		snprintf(info, infolen, "%s%%%s@", methodname, authname);
+
+	return info;
+}
+
+
 static struct rule_t *
-addrule(newrule, rulebase)
+addrule(newrule, rulebase, client)
 	const struct rule_t *newrule;
 	struct rule_t **rulebase;
+	int client;
 {
 	static const struct serverstate_t state;
 	const char *function = "addrule()";
 	struct rule_t *rule;
+	int i;
 
 	if ((rule = malloc(sizeof(*rule))) == NULL)
-		serrx(1, "%s: %s", function, NOMEM);
+		serrx(EXIT_FAILURE, "%s: %s", function, NOMEM);
 	*rule = *newrule;
 
 	/* try to set values not set to a sensible default. */
@@ -888,25 +908,68 @@ addrule(newrule, rulebase)
 		memset(&rule->state.command, UCHAR_MAX, sizeof(rule->state.command));
 
 	/*
-	 * If no method set, set all we support.  This in practice
-	 * limits the methods we accept here to the globally set methods
-	 * (config.methodv), since they are checked before we get to
-	 * rulespesific checks.  We can't just copy config.methodv
-	 * since it may not be set yet.
+	 * If no method set, set all set in the global config.methodv.
 	 */
 	if (rule->state.methodc == 0) {
 		int *methodv = rule->state.methodv;
 		int *methodc = &rule->state.methodc;
 
-		if (rule->user == NULL)
-			methodv[(*methodc)++] = AUTHMETHOD_NONE;
+		for (i = 0; i < config.methodc; ++i) {
+			int addmethod = 0;
 
-		methodv[(*methodc)++] = AUTHMETHOD_UNAME;
-#if HAVE_LIBWRAP
-		methodv[(*methodc)++] = AUTHMETHOD_RFC931;
-#endif
+			if (client) {
+				switch (config.methodv[i]) {
+					case AUTHMETHOD_NONE:
+						if (rule->user != NULL)
+							break; /* can't get username with this method. */
+						addmethod = 1;
+						break;
+
+					case AUTHMETHOD_RFC931:
+						addmethod = 1;
+						break;
+
+					default:  /* some non-client method presumably. */
+						break;
+				}
+			}
+			else {
+				switch (config.methodv[i]) {
+					case AUTHMETHOD_NONE:
+						if (rule->user != NULL)
+							break; /* can't get username with this method. */
+						addmethod = 1;
+						break;
+
+					case AUTHMETHOD_UNAME:
+					case AUTHMETHOD_RFC931:
+						addmethod = 1;
+						break;
+
+					default:
+						SERRX(config.methodv[i]);
+				}
+			}
+
+			if (addmethod)
+				methodv[(*methodc)++] = config.methodv[i];
+		}
 	}
 
+	/* warn about methods not set in the global method?  May not be an error. */
+	for (i = 0; i < rule->state.methodc; ++i)
+		if (!methodisset(rule->state.methodv[i], (const int *)&config.methodv,
+		(size_t)config.methodc)) {
+			char buf[256];
+
+			snprintf(buf, sizeof(buf), "method \"%s\" set in rule but not "
+			"in global method",
+			method2string(rule->state.methodv[i]));
+			yywarn(buf);
+
+		}
+
+
 	/* if no protocol set, set all. */
 	if (memcmp(&state.protocol, &rule->state.protocol, sizeof(state.protocol))
 	== 0)
@@ -948,9 +1011,8 @@ checkrule(rule)
 	const struct rule_t *rule;
 {
 
-	if (rule->user != NULL)
-		if (rule->state.methodc == 0)
-			yyerror("rule restricts by name, but no username-based method given");
+	if (rule->state.methodc == 0)
+		yywarn("rule allows no methods");
 }
 
 static void
@@ -1015,14 +1077,13 @@ libwrapinit(s, request)
 #endif /* HAVE_LIBWRAP */
 
 static int
-connectisok(request, rule, state)
+connectisok(request, rule)
 #if HAVE_LIBWRAP
 	struct request_info *request;
 #else
 	void *request;
 #endif
 	const struct rule_t *rule;
-	struct connectionstate_t *state;
 {
 
 #if HAVE_LIBWRAP
@@ -1034,7 +1095,6 @@ connectisok(request, rule, state)
 		const char *function = "connectisok()";
 		char libwrap[LIBWRAPBUF];
 		uid_t euid;
-		int checkforname;
 
 		socks_seteuid(&euid, config.uid.libwrap);
 
@@ -1050,34 +1110,6 @@ connectisok(request, rule, state)
 		}
 		process_options(libwrap, request);
 
-		/*
-		 * check if we got a username and won't clobber anything by saving it.
-		 */
-		switch (state->auth.method) {
-			case AUTHMETHOD_NONE:	/* doesn't take any memory from rfc931. */
-				checkforname = 1;
-				break;
-
-			default:
-				checkforname = 0;	/* can't take it or should already have it. */
-		}
-
-		if (checkforname) {
-			/* XXX can't use eval_user() since it always does rfc931 lookup. */
-			if (*request->user != NUL) {
-				strncpy(state->auth.mdata.rfc931.name, request->user,
-				sizeof(state->auth.mdata.rfc931.name) - 1);
-
-				if (state->auth.mdata.rfc931.name
-				[sizeof(state->auth.mdata.rfc931.name) - 1] != NUL) {
-					slog(LOG_DEBUG, "%s: rfc931 name too long, truncated", function);
-					state->auth.mdata.rfc931.name
-					[sizeof(state->auth.mdata.rfc931.name)
-					- 1] = NUL;
-				}
-			}
-		}
-
 		if (config.srchost.nounknown)
 			if (strcmp(eval_hostname(request->client), STRING_UNKNOWN) == 0) {
 				slog(LOG_INFO, "%s: srchost unknown",
