@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd.c,v 1.248 1999/12/20 13:07:42 karls Exp $";
+"$Id: sockd.c,v 1.256 2000/05/31 12:14:55 karls Exp $";
 
 	/*
 	 * signal handlers
@@ -136,7 +136,10 @@ main(argc, argv, envp)
 #endif /* HAVE_SETPROCTITLE */
 
 	const int exitsignalv[] = {
-		SIGINT, SIGQUIT, SIGBUS, SIGSEGV, SIGTERM
+		SIGINT, SIGQUIT, SIGBUS, SIGSEGV, SIGTERM, SIGILL, SIGFPE
+#ifdef SIGSYS
+		, SIGSYS
+#endif /* SIGSYS */
 	};
 	const size_t exitsignalc = ELEMENTS(exitsignalv);
 
@@ -182,7 +185,7 @@ main(argc, argv, envp)
 
 		close(p);
 	}
-	initlog(); /* for syslog. */
+	newprocinit();
 
 	/*
 	 * Check system limits against what we need.
@@ -287,7 +290,7 @@ main(argc, argv, envp)
 		if ((pid = fork()) == -1)
 			swarn("fork()");
 		else if (pid == 0) {
-			config.state.pid = getpid();
+			newprocinit();
 			break;
 		}
 		else
@@ -336,7 +339,7 @@ main(argc, argv, envp)
 			char command;
 			int childisbad = 0;
 
-			if ((p = readn(child->ack, &command, sizeof(command)))
+			if ((p = readn(child->ack, &command, sizeof(command), NULL))
 			!= sizeof(command)) {
 				swarn("readn(child->ack) from %schild %lu failed",
 				childtype2string(child->type), (unsigned long)child->pid);
@@ -497,7 +500,7 @@ main(argc, argv, envp)
 						case EWOULDBLOCK:		/* BSD */
 						case ECONNABORTED:	/* POSIX */
 
-						/* rest appears to be linux stuff according to apache src. */
+						/* rest appears to be Linux stuff according to apache src. */
 #ifdef ECONNRESET
 						case ECONNRESET:
 #endif
@@ -539,11 +542,11 @@ main(argc, argv, envp)
 
 #if HAVE_LINUX_BUGS
 				/*
-				 * yes, linux manages to lose the descriptor flags, workaround
+				 * yes, Linux manages to lose the descriptor flags, workaround
 				 *	might be insufficient.
 				 */
 				if (fcntl(client, F_SETFL, fcntl(l->s, F_GETFL, 0)) != 0)
-					swarn("tried to work around linux bug via fcntl()");
+					swarn("tried to work around Linux bug via fcntl()");
 #endif /* HAVE_LINUX_BUGS */
 
 				++config.stat.accepted;
@@ -632,7 +635,7 @@ showlicense(void)
 	printf("%s: %s v%s\n%s\n", __progname, PACKAGE, VERSION,
 "\
 /*\n\
- * Copyright (c) 1997, 1998, 1999\n\
+ * Copyright (c) 1997, 1998, 1999, 2000\n\
  *      Inferno Nettverk A/S, Norway.  All rights reserved.\n\
  *\n\
  * Redistribution and use in source and binary forms, with or without\n\
@@ -718,7 +721,7 @@ serverinit(argc, argv, envp)
 
 			case 'N':
 				if ((config.option.serverc = atoi(optarg)) < 1)
-					serrx(1, "%s: illegal value for -%c: %d",
+					serrx(EXIT_FAILURE, "%s: illegal value for -%c: %d",
 					function, ch, config.option.serverc);
 				break;
 
@@ -765,7 +768,8 @@ serverinit(argc, argv, envp)
 		if (daemon(1, 0) != 0)
 			serr(EXIT_FAILURE, "daemon()");
 
-	config.state.pid = getpid();
+	newprocinit();
+
 	if ((config.state.motherpidv
 	= (pid_t *)malloc(sizeof(*config.state.motherpidv) * config.option.serverc))
 	== NULL)
@@ -853,7 +857,7 @@ checksettings(void)
 
 			if (bind(s, (struct sockaddr *)&config.externalv[i],
 			sizeof(config.externalv[i])) != 0)
-				serrx(EXIT_FAILURE, "%s: can't bind external address: %s",
+				serr(EXIT_FAILURE, "%s: can't bind external address: %s",
 				function, sockaddr2string((struct sockaddr *)&config.externalv[i],
 				addrstring, sizeof(addrstring)));
 			close(s);
@@ -1009,7 +1013,7 @@ sigchld(sig)
 	}
 
 	/*
-	 * If we get alot of childdeaths in a short time, assume something
+	 * If we get a lot of childdeaths in a short time, assume something
 	 * is wrong.
 	 */
 
