@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1997, 1998, 1999
+ * Copyright (c) 1997, 1998, 1999, 2000
  *      Inferno Nettverk A/S, Norway.  All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -44,7 +44,7 @@
 #include "common.h"
 
 static const char rcsid[] =
-"$Id: sockd_request.c,v 1.110 1999/12/22 09:29:27 karls Exp $";
+"$Id: sockd_request.c,v 1.119 2000/06/09 10:45:20 karls Exp $";
 
 /*
  * Since it only handles one client at a time there is no possibility
@@ -142,7 +142,8 @@ run_request(mother)
 
 		dorequest(mother->s, &req);
 
-		if (writen(mother->ack, &command, sizeof(command)) != sizeof(command))
+		if (writen(mother->ack, &command, sizeof(command), NULL)
+		!= sizeof(command))
 			serr(EXIT_FAILURE, "%s: sending ack to mother failed", function);
 
 #if DIAGNOSTIC
@@ -323,36 +324,40 @@ dorequest(mother, request)
 	 * packet looks ok, fill in remaining bits needed to check rules.
 	 */
 
+	/* LINTED pointer casts may be troublesome */
+	sockaddr2sockshost((const struct sockaddr *)&request->from,
+	&io.control.host);
+
 	switch (request->req.command) {
 		case SOCKS_BIND:
-			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((const struct sockaddr *)&request->from, &io.src);
-			io.dst = request->req.host;
+			io.src.host = io.control.host;
 
-			if (io.dst.atype					!= SOCKS_ADDR_IPV4
-			||  io.dst.addr.ipv4.s_addr	!= htonl(0)
-			||  io.dst.port					== htons(0))
+			io.dst.host = request->req.host;
+
+			if (io.dst.host.atype					!= SOCKS_ADDR_IPV4
+			||  io.dst.host.addr.ipv4.s_addr		!= htonl(0)
+			||  io.dst.host.port						== htons(0))
 				io.state.extension.bind = 0;	/* not requesting bind extension. */
 			break;
 
 		case SOCKS_CONNECT:
-			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((const struct sockaddr *)&request->from, &io.src);
-			io.dst = request->req.host;
+			io.src.host = io.control.host;
+			io.dst.host = request->req.host;
 			break;
 
 		case SOCKS_UDPASSOCIATE:
 			/*
-			 * for UDP_ASSOCIATE we are getting clients udp address,
+			 * for UDP_ASSOCIATE we are getting clients UDP address,
 			 * not destination in request.
 			 * Destination address will be checked in the i/o loop for
 			 * each destination, for now just set it to INADDR_ANY.
 			 */
 
-			io.src						= request->req.host;
-			io.dst.atype				= SOCKS_ADDR_IPV4;
-			io.dst.addr.ipv4.s_addr	= htonl(INADDR_ANY);
-			io.dst.port					= htons(0);
+			io.src.host							= request->req.host;
+
+			io.dst.host.atype					= SOCKS_ADDR_IPV4;
+			io.dst.host.addr.ipv4.s_addr	= htonl(INADDR_ANY);
+			io.dst.host.port					= htons(0);
 			break;
 
 		default:
@@ -390,14 +395,14 @@ dorequest(mother, request)
 			switch (request->req.version) {
 				case SOCKS_V4:
 					if (io.state.extension.bind)
-						bound.sin_port	= io.dst.port;
+						bound.sin_port	= io.dst.host.port;
 					else
 						/* best we can try for is to use same port as source. */
 						bound.sin_port	= request->from.sin_port;
 					break;
 
 				case SOCKS_V5:
-					bound.sin_port = io.dst.port;
+					bound.sin_port = io.dst.host.port;
 					break;
 
 				default:
@@ -455,6 +460,7 @@ dorequest(mother, request)
 		return;
 	}
 
+
 	/* rules permit? */
 	switch (request->req.command) {
 		case SOCKS_BIND: {
@@ -472,19 +478,24 @@ dorequest(mother, request)
 			/* LINTED pointer casts may be troublesome */
 			sockaddr2sockshost((struct sockaddr *)&bound, &boundhost);
 
-			permit
-			= rulespermit(request->s, &io.rule, &io.state, &io.src, &boundhost);
-			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src, &boundhost,
-			NULL, 0);
+			permit = rulespermit(request->s, &io.rule, &io.state, &io.src.host,
+			&boundhost);
 
+			io.src.auth = io.control.auth = io.state.auth;
+
+			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
+			&io.src.auth, &boundhost, &io.dst.auth, NULL, 0);
 			break;
 		}
 
 		case SOCKS_CONNECT:
-			permit
-			= rulespermit(request->s, &io.rule, &io.state, &io.src, &io.dst);
-			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src, &io.dst,
-			NULL, 0);
+			permit = rulespermit(request->s, &io.rule, &io.state, &io.src.host,
+			&io.dst.host);
+
+			io.src.auth = io.control.auth = io.state.auth;
+
+			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
+			&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
 			break;
 
 		case SOCKS_UDPASSOCIATE: {
@@ -494,13 +505,14 @@ dorequest(mother, request)
 			/*
 			 * Client is allowed to send a "incomplete" address.
 			 */
-			if (io.src.atype == SOCKS_ADDR_IPV4
-			&& (io.src.addr.ipv4.s_addr == htonl(0) || io.src.port == htons(0)))
+			if (io.src.host.atype == SOCKS_ADDR_IPV4
+			&& (io.src.host.addr.ipv4.s_addr == htonl(0)
+			  || io.src.host.port == htons(0)))
 				src = NULL;
 			else
-				src = &io.src;
+				src = &io.src.host;
 
-			/* only set temporary here for one replypacket at a time. */
+			/* check for i/o both ways. */
 			replystate				= io.state;
 			replystate.command	= SOCKS_UDPREPLY;
 
@@ -508,8 +520,10 @@ dorequest(mother, request)
 			permit = rulespermit(request->s, &io.rule, &io.state, src, NULL)
 					|| rulespermit(request->s, &io.rule, &replystate, NULL, src);
 
-			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src, &io.dst, NULL,
-			0);
+			io.src.auth = io.control.auth = io.state.auth;
+
+			iolog(&io.rule, &io.state, OPERATION_CONNECT, &io.src.host,
+			&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
 			break;
 		}
 
@@ -527,6 +541,9 @@ dorequest(mother, request)
 	/*
 	 * Set up missing bits of io and send it to mother.
 	 */
+
+	io.dst.auth.method	= AUTHMETHOD_NONE; /* no remote auth so far. */
+
 	switch (io.state.command) {
 		case SOCKS_BIND: {
 			struct sockd_io_t *iolist;
@@ -598,13 +615,13 @@ dorequest(mother, request)
 				sv[ourpipe]		= pipev[1];
 
 				/* LINTED pointer casts may be troublesome */
-				((struct sockaddr_in *)&clientaddr)->sin_port = io.dst.port;
+				((struct sockaddr_in *)&clientaddr)->sin_port = io.dst.host.port;
 			}
 
 			/* let client know what address we bound to on it's behalf. */
 			if (send_response(sv[client], &response) != 0) {
-				iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src, &response.host,
-				NULL, 0);
+				iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src.host,
+				&io.src.auth, &response.host, &io.dst.auth, NULL, 0);
 				closev(sv, ELEMENTS(sv));
 				break;
 			}
@@ -617,7 +634,7 @@ dorequest(mother, request)
 			 * regardless of what kind of bind semantics are in use,
 			 * portnumber is something we ignore when checking remote peer.
 			 */
-			io.dst.port = htons(0);
+			io.dst.host.port = htons(0);
 
 			emfile = 0;
 			iolist = NULL;
@@ -665,8 +682,8 @@ dorequest(mother, request)
 					query.auth = request->req.auth;
 					switch (p = recv_sockspacket(sv[client], &query, &state)) {
 						case -1:
-							iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src,
-							&response.host, NULL, 0);
+							iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src.host,
+							&io.src.auth, &response.host, &io.dst.auth, NULL, 0);
 							break;
 
 						case 0:
@@ -703,10 +720,10 @@ dorequest(mother, request)
 
 								/* LINTED pointer casts may be troublesome */
 								SASSERTX(sockaddrareeq((struct sockaddr *)
-								&fio->in.laddr, &queryaddr));
+								&fio->dst.laddr, &queryaddr));
 
 								/* LINTED pointer casts may be troublesome */
-								sockaddr2sockshost((struct sockaddr *)&fio->out.raddr,
+								sockaddr2sockshost((struct sockaddr *)&fio->src.raddr,
 								&queryresponse.host);
 							}
 
@@ -719,8 +736,9 @@ dorequest(mother, request)
 							else
 								if ((p = send_response(sv[client], &queryresponse))
 								!= 0)
-									iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src,
-									&response.host, NULL, 0);
+									iolog(&io.rule, &io.state, OPERATION_ABORT,
+									&io.src.host, &io.src.auth, &response.host,
+									&io.dst.auth, NULL, 0);
 						}
 					}
 
@@ -742,7 +760,7 @@ dorequest(mother, request)
 						case EWOULDBLOCK:		/* BSD */
 						case ECONNABORTED:	/* POSIX */
 
-						/* rest appears to be linux stuff according to apache src. */
+						/* rest appears to be Linux stuff according to apache src. */
 #ifdef ECONNRESET
 						case ECONNRESET:
 #endif
@@ -768,24 +786,30 @@ dorequest(mother, request)
 
 				/* accepted connection; does remote address match requested? */
 				if (io.state.extension.bind
-				|| addressmatch(sockshost2ruleaddress(&io.dst, &ruleaddr),
+				|| addressmatch(sockshost2ruleaddress(&io.dst.host, &ruleaddr),
 				&remotehost, SOCKS_TCP, 1)) {
 					bindio						= io; /* quick init of most stuff. */
-					bindio.src					= remotehost;
-					sockaddr2sockshost(&clientaddr, &bindio.dst);
+					bindio.src.host			= remotehost;
+					sockaddr2sockshost(&clientaddr, &bindio.dst.host);
 					bindio.state.command		= SOCKS_BINDREPLY;
+					bindio.dst.auth			= io.src.auth;
 
-					permit = rulespermit(sv[client], &bindio.rule, &bindio.state,
-					&bindio.src, &bindio.dst);
+					bindio.state.auth.method = AUTHMETHOD_NONE;
+
+					permit = rulespermit(sv[remote], &bindio.rule, &bindio.state,
+					&bindio.src.host, &bindio.dst.host);
+
+					bindio.src.auth = bindio.state.auth;
 
 					iolog(&bindio.rule, &bindio.state, OPERATION_CONNECT,
-					&bindio.src, &bindio.dst, NULL, 0);
+					&bindio.src.host, &bindio.src.auth, &bindio.dst.host,
+					&bindio.dst.auth, NULL, 0);
 
 				}
 				else {
 					slog(LOG_INFO, "%s(0): unexpected bindreply: %s -> %s",
 					VERDICT_BLOCKs, sockaddr2string(&remoteaddr, a, sizeof(a)),
-					sockshost2string(&io.src, b, sizeof(b)));
+					sockshost2string(&io.src.host, b, sizeof(b)));
 					permit = 0;
 				}
 
@@ -850,7 +874,8 @@ dorequest(mother, request)
 
 					if (connect(sv[reply], &clientaddr, sizeof(clientaddr)) != 0) {
 						iolog(&bindio.rule, &bindio.state, OPERATION_ABORT,
-						&bindio.src, &bindio.dst, NULL, 0);
+						&bindio.src.host, &bindio.src.auth, &bindio.dst.host,
+						&bindio.dst.auth, NULL, 0);
 						break;
 					}
 				}
@@ -879,7 +904,6 @@ dorequest(mother, request)
 
 				bindio.control.laddr	= request->to;
 				bindio.control.raddr	= request->from;
-				bindio.control.state	= bindio.state;
 
 				/* back to blocking. */
 				if (fcntl(sv[remote], F_SETFL, flags) == -1) {
@@ -888,31 +912,31 @@ dorequest(mother, request)
 				}
 
 				if (bindio.state.extension.bind) {
-					bindio.in.s			= sv[reply];
-					bindio.in.laddr	= replyaddr;
-					bindio.in.state	= bindio.state;
+					bindio.dst.s		= sv[reply];
+					bindio.dst.laddr	= replyaddr;
 				}
 				else {
-					bindio.in			= bindio.control;
-					bindio.in.laddr	= request->from;
+					bindio.dst			= bindio.control;
+					bindio.dst.laddr	= request->from;
 				}
 				/* LINTED pointer casts may be troublesome */
-				bindio.in.raddr					= *(struct sockaddr_in *)&clientaddr;
+				bindio.dst.raddr		= *(struct sockaddr_in *)&clientaddr;
 
-				bindio.out.s						= sv[remote];
+				bindio.src.s			= sv[remote];
 				/* LINTED pointer casts may be troublesome */
-				bindio.out.laddr					= *(struct sockaddr_in *)&boundaddr;
+				bindio.src.laddr	= *(struct sockaddr_in *)&boundaddr;
 				/* LINTED pointer casts may be troublesome */
-				bindio.out.raddr					= *(struct sockaddr_in *)&remoteaddr;
-				bindio.out.state.auth.method	= AUTHMETHOD_NONE;
+				bindio.src.raddr	= *(struct sockaddr_in *)&remoteaddr;
 
 				if (bindio.state.extension.bind)
 					/* add to list, client will query. */
 					iolist = io_add(iolist, &bindio);
 				else {
-					response.host = bindio.dst;
+					response.host = bindio.dst.host;
 					flushio(mother, sv[client], &response, &bindio);
-					sv[client] = sv[remote] = -1; /* flushio() closes; closev(). */
+
+					/* flushio() closes these, not closev(). */
+					sv[client] = sv[remote] = -1;
 					break;	/* only one connection to relay and that is done. */
 				}
 			}
@@ -934,45 +958,45 @@ dorequest(mother, request)
 		case SOCKS_CONNECT: {
 			socklen_t sinlen;
 
-			if (socks_connect(out, &io.dst) != 0) {
-				iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src, &io.dst,
-				NULL, 0);
+			if (socks_connect(out, &io.dst.host) != 0) {
+				iolog(&io.rule, &io.state, OPERATION_ABORT, &io.src.host,
+				&io.src.auth, &io.dst.host, &io.dst.auth, NULL, 0);
 				send_failure(request->s, &response,
 				errno2reply(errno, response.version));
 				close(request->s);
 				break;
 			}
 
-			io.in.s			= request->s;
-			io.in.laddr		= request->to;
-			io.in.raddr		= request->from;
-			io.in.state		= io.state;
+			io.control.s		= request->s;
+			io.control.laddr	= request->to;
+			io.control.raddr	= request->from;
 
-			io.out.s			= out;
-			io.out.state	= io.state;
-			sinlen			= sizeof(io.out.raddr);
+			io.src	= io.control;
+
+			io.dst.s	= out;
+			sinlen	= sizeof(io.dst.raddr);
 			/* LINTED pointer casts may be troublesome */
-			if (getpeername(io.out.s, (struct sockaddr *)&io.out.raddr, &sinlen)
+			if (getpeername(io.dst.s, (struct sockaddr *)&io.dst.raddr, &sinlen)
 			!= 0) {
-				swarn("%s: getpeername(io.out.s)", function);
+				swarn("%s: getpeername(io.dst.s)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
 				break;
 			}
 
-			sinlen = sizeof(io.out.laddr);
+			sinlen = sizeof(io.dst.laddr);
 			/* LINTED pointer casts may be troublesome */
-			if (getsockname(io.out.s, (struct sockaddr *)&io.out.laddr, &sinlen)
+			if (getsockname(io.dst.s, (struct sockaddr *)&io.dst.laddr, &sinlen)
 			!= 0) {
-				swarn("%s: getsockname(io.out.s)", function);
+				swarn("%s: getsockname(io.dst.s)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
 				break;
 			}
 
 			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((struct sockaddr *)&io.out.laddr, &response.host);
-			response.reply	= (char)sockscode(response.version, SOCKS_SUCCESS);
+			sockaddr2sockshost((struct sockaddr *)&io.dst.laddr, &response.host);
+			response.reply	= sockscode(response.version, SOCKS_SUCCESS);
 
 			flushio(mother, request->s, &response, &io);
 			break;
@@ -995,30 +1019,33 @@ dorequest(mother, request)
 			/* LINTED pointer casts may be troublesome */
 			sockshost2sockaddr(&request->req.host, (struct sockaddr *)&client);
 
-			io.in.s					= clientfd;
-			io.in.state				= io.state;
-			io.in.raddr				= client;
-			io.in.laddr				= request->to;
-			io.in.laddr.sin_port = htons(0);
+			io.control.s			= request->s;
+			io.control.laddr		= request->from;
+			io.control.raddr		= request->to;
+
+			io.src.s					= clientfd;
+			io.src.raddr				= client;
+			io.src.laddr				= request->to;
+			io.src.laddr.sin_port = htons(0);
 
 			/*
-			 * bind to address for receiving udp packets so we can tell client
-			 * where to send its packets.
+			 * bind address for receiving UDP packets so we can tell client
+			 * where to send it's packets.
 			 */
 			/* LINTED pointer casts may be troublesome */
-			if (sockd_bind(clientfd, (struct sockaddr *)&io.in.laddr, 0) != 0) {
+			if (sockd_bind(clientfd, (struct sockaddr *)&io.src.laddr, 0) != 0) {
 				/* LINTED pointer casts may be troublesome */
 				swarn("%s: bind(%s)", function,
-				sockaddr2string((struct sockaddr *)&io.in.laddr, a, sizeof(a)));
+				sockaddr2string((struct sockaddr *)&io.src.laddr, a, sizeof(a)));
 				send_failure(request->s, &response, SOCKS_FAILURE);
 				close(request->s);
 				close(clientfd);
 				break;
 			}
 
-			boundlen = sizeof(io.in.laddr);
+			boundlen = sizeof(io.src.laddr);
 			/* LINTED pointer casts may be troublesome */
-			if (getsockname(clientfd, (struct sockaddr *)&io.in.laddr, &boundlen)
+			if (getsockname(clientfd, (struct sockaddr *)&io.src.laddr, &boundlen)
 			!= 0) {
 				swarn("%s: getsockname(clientfd)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
@@ -1027,13 +1054,10 @@ dorequest(mother, request)
 				break;
 			}
 
-			io.out.s							= out;
-			io.out.state					= io.state;
-			io.out.state.auth.method	= AUTHMETHOD_NONE;
-
-			boundlen = sizeof(io.out.laddr);
+			io.dst.s					= out;
+			boundlen = sizeof(io.dst.laddr);
 			/* LINTED pointer casts may be troublesome */
-			if (getsockname(out, (struct sockaddr *)&io.out.laddr, &boundlen)
+			if (getsockname(out, (struct sockaddr *)&io.dst.laddr, &boundlen)
 			!= 0) {
 				swarn("%s: getsockname(out)", function);
 				send_failure(request->s, &response, SOCKS_FAILURE);
@@ -1041,24 +1065,18 @@ dorequest(mother, request)
 				close(clientfd);
 				break;
 			}
-
 			/* remote out changes each time, set to INADDR_ANY for now. */
-			bzero(&io.out.raddr, sizeof(io.out.raddr));
-			io.out.raddr.sin_family			= AF_INET;
-			io.out.raddr.sin_addr.s_addr	= htonl(INADDR_ANY);
-			io.out.raddr.sin_port			= htons(0);
-
-			io.control.s						= request->s;
-			io.control.laddr					= request->from;
-			io.control.raddr					= request->to;
-			io.control.state					= io.state;
+			bzero(&io.dst.raddr, sizeof(io.dst.raddr));
+			io.dst.raddr.sin_family			= AF_INET;
+			io.dst.raddr.sin_addr.s_addr	= htonl(INADDR_ANY);
+			io.dst.raddr.sin_port			= htons(0);
 
 			if (request->req.flag & SOCKS_USECLIENTPORT)
-				if (client.sin_port == io.out.laddr.sin_port)
+				if (client.sin_port == io.dst.laddr.sin_port)
 					response.flag |= SOCKS_USECLIENTPORT;
 
 			/* LINTED pointer casts may be troublesome */
-			sockaddr2sockshost((struct sockaddr *)&io.in.laddr, &response.host);
+			sockaddr2sockshost((struct sockaddr *)&io.src.laddr, &response.host);
 			response.reply	= (char)sockscode(response.version, SOCKS_SUCCESS);
 			flushio(mother, request->s, &response, &io);
 			break;
@@ -1099,36 +1117,36 @@ flushio(mother, clientcontrol, response, io)
 
 #if SOCKD_IOMAX == 1
 	/* only one client per process; doesn't matter much whether we block. */
-	io->in.sndlowat	= sndlowat;
-	io->out.sndlowat	= sndlowat;
+	io->src.sndlowat	= sndlowat;
+	io->dst.sndlowat	= sndlowat;
 
 #elif	HAVE_SO_SNDLOWAT
 
 	len = sizeof(value);
-	if (getsockopt(io->in.s, SOL_SOCKET, SO_SNDBUF, &value, &len) != 0)
+	if (getsockopt(io->src.s, SOL_SOCKET, SO_SNDBUF, &value, &len) != 0)
 		swarn("%s: getsockopt(in, SO_SNDBUF)", function);
 	sndlowat = MIN(sndlowat, value * skew);
 
-	if (setsockopt(io->in.s, SOL_SOCKET, SO_SNDLOWAT, &sndlowat,
+	if (setsockopt(io->src.s, SOL_SOCKET, SO_SNDLOWAT, &sndlowat,
 	sizeof(sndlowat)) != 0)
 		swarn("%s: setsockopt(in, SO_SNDLOWAT)", function);
 
-	len = sizeof(io->in.sndlowat);
-	if (getsockopt(io->in.s, SOL_SOCKET, SO_SNDLOWAT, &io->in.sndlowat, &len)
+	len = sizeof(io->src.sndlowat);
+	if (getsockopt(io->src.s, SOL_SOCKET, SO_SNDLOWAT, &io->src.sndlowat, &len)
 	!= 0)
 		swarn("%s: getsockopt(in, SO_SNDLOWAT)", function);
 
 	len = sizeof(value);
-	if (getsockopt(io->out.s, SOL_SOCKET, SO_SNDBUF, &value, &len) != 0)
+	if (getsockopt(io->dst.s, SOL_SOCKET, SO_SNDBUF, &value, &len) != 0)
 		swarn("%s: getsockopt(out, SO_SNDBUF)", function);
 	sndlowat = MIN(sndlowat, value * skew);
 
-	if (setsockopt(io->out.s, SOL_SOCKET, SO_SNDLOWAT, &sndlowat,
+	if (setsockopt(io->dst.s, SOL_SOCKET, SO_SNDLOWAT, &sndlowat,
 	sizeof(sndlowat)) != 0)
 		swarn("%s: setsockopt(out, SO_SNDLOWAT", function);
 
-	len = sizeof(io->out.sndlowat);
-	if (getsockopt(io->in.s, SOL_SOCKET, SO_SNDLOWAT, &io->out.sndlowat, &len)
+	len = sizeof(io->dst.sndlowat);
+	if (getsockopt(io->src.s, SOL_SOCKET, SO_SNDLOWAT, &io->dst.sndlowat, &len)
 	!= 0)
 		swarn("%s: getsockopt(in, SO_SNDLOWAT", function);
 
@@ -1136,37 +1154,38 @@ flushio(mother, clientcontrol, response, io)
 	switch (io->state.command) {
 		case SOCKS_UDPASSOCIATE:
 			len = sizeof(sndlowat);
-			if (getsockopt(io->in.s, SOL_SOCKET, SO_SNDBUF, &sndlowat, &len) != 0){
-				swarn("%s: getsockopt(SO_SNDBUF", function);
-				io->in.sndlowat = SOCKD_BUFSIZEUDP;
-			}
-			else if (sndlowat == 0)
-				io->in.sndlowat = SOCKD_BUFSIZEUDP;
-			else
-				io->in.sndlowat = sndlowat;
-
-			len = sizeof(sndlowat);
-			if (getsockopt(io->out.s, SOL_SOCKET, SO_SNDBUF, &sndlowat, &len)
+			if (getsockopt(io->src.s, SOL_SOCKET, SO_SNDBUF, &sndlowat, &len)
 			!= 0) {
 				swarn("%s: getsockopt(SO_SNDBUF", function);
-				io->out.sndlowat = SOCKD_BUFSIZEUDP;
+				io->src.sndlowat = SOCKD_BUFSIZEUDP;
 			}
 			else if (sndlowat == 0)
-				io->out.sndlowat = SOCKD_BUFSIZEUDP;
+				io->src.sndlowat = SOCKD_BUFSIZEUDP;
 			else
-				io->out.sndlowat = sndlowat;
+				io->src.sndlowat = sndlowat;
+
+			len = sizeof(sndlowat);
+			if (getsockopt(io->dst.s, SOL_SOCKET, SO_SNDBUF, &sndlowat, &len)
+			!= 0) {
+				swarn("%s: getsockopt(SO_SNDBUF", function);
+				io->dst.sndlowat = SOCKD_BUFSIZEUDP;
+			}
+			else if (sndlowat == 0)
+				io->dst.sndlowat = SOCKD_BUFSIZEUDP;
+			else
+				io->dst.sndlowat = sndlowat;
 
 			break;
 
 		default:
 			/* TCP; use minimum guess. */
-			io->in.sndlowat	= SO_SNDLOWAT_SIZE;
-			io->out.sndlowat	= SO_SNDLOWAT_SIZE;
+			io->src.sndlowat	= SO_SNDLOWAT_SIZE;
+			io->dst.sndlowat	= SO_SNDLOWAT_SIZE;
 	}
 #endif  /* SOCKD_IOMAX > 1 && !HAVE_SO_SNDLOWAT */
 
-	SASSERTX(io->in.sndlowat > 0
-	&& io->out.sndlowat >= sizeof(struct udpheader_t));
+	SASSERTX(io->src.sndlowat > 0
+	&& io->dst.sndlowat >= sizeof(struct udpheader_t));
 
 	if (send_response(clientcontrol, response) == 0)
 		if (send_io(mother, io) != 0)
@@ -1261,8 +1280,8 @@ io_find(iolist, addr)
 	io = iolist;
 	while (io != NULL)
 		/* LINTED pointer casts may be troublesome */
-		if (sockaddrareeq((struct sockaddr *)&io->in.laddr, addr)
-		||  sockaddrareeq((struct sockaddr *)&io->out.laddr, addr)
+		if (sockaddrareeq((struct sockaddr *)&io->src.laddr, addr)
+		||  sockaddrareeq((struct sockaddr *)&io->dst.laddr, addr)
 		||  sockaddrareeq((struct sockaddr *)&io->control.laddr, addr))
 			return io;
 		else
